@@ -98,6 +98,7 @@ class YoutubeDLDownloader(DownloaderBase):
                 pathfmt.realdirectory + filename)
         else:
             pathfmt.set_extension(info_dict["ext"])
+            pathfmt.build_path()
 
         if pathfmt.exists():
             pathfmt.temppath = ""
@@ -118,6 +119,7 @@ class YoutubeDLDownloader(DownloaderBase):
 
     def _download_playlist(self, ytdl_instance, pathfmt, info_dict):
         pathfmt.set_extension("%(playlist_index)s.%(ext)s")
+        pathfmt.build_path()
         self._set_outtmpl(ytdl_instance, pathfmt.realpath)
 
         for entry in info_dict["entries"]:
@@ -138,9 +140,14 @@ class YoutubeDLDownloader(DownloaderBase):
     @staticmethod
     def _set_outtmpl(ytdl_instance, outtmpl):
         try:
-            ytdl_instance.outtmpl_dict["default"] = outtmpl
+            ytdl_instance._parse_outtmpl
         except AttributeError:
-            ytdl_instance.params["outtmpl"] = outtmpl
+            try:
+                ytdl_instance.outtmpl_dict["default"] = outtmpl
+            except AttributeError:
+                ytdl_instance.params["outtmpl"] = outtmpl
+        else:
+            ytdl_instance.params["outtmpl"] = {"default": outtmpl}
 
 
 def compatible_formats(formats):
