@@ -10,12 +10,13 @@ import re
 
 modules = [
     "2chan",
+    "2chen",
     "35photo",
     "3dbooru",
     "420chan",
     "4chan",
     "500px",
-    "8kun",
+    "8chan",
     "8muses",
     "adultempire",
     "architizer",
@@ -25,6 +26,8 @@ modules = [
     "bcy",
     "behance",
     "blogger",
+    "bunkr",
+    "catbox",
     "comicvine",
     "cyberdrop",
     "danbooru",
@@ -35,7 +38,10 @@ modules = [
     "exhentai",
     "fallenangels",
     "fanbox",
+    "fanleaks",
     "fantia",
+    "fapello",
+    "fapachi",
     "flickr",
     "furaffinity",
     "fuskator",
@@ -53,6 +59,7 @@ modules = [
     "hentaihere",
     "hiperdex",
     "hitomi",
+    "hotleak",
     "idolcomplex",
     "imagebam",
     "imagechest",
@@ -64,17 +71,18 @@ modules = [
     "inkbunny",
     "instagram",
     "issuu",
+    "itaku",
     "kabeuchi",
     "keenspot",
     "kemonoparty",
     "khinsider",
-    "kissgoddess",
-    "kohlchan",
     "komikcast",
+    "lexica",
     "lightroom",
     "lineblog",
     "livedoor",
     "luscious",
+    "lynxchan",
     "mangadex",
     "mangafox",
     "mangahere",
@@ -86,14 +94,16 @@ modules = [
     "mememuseum",
     "myhentaigallery",
     "myportfolio",
+    "nana",
     "naver",
     "naverwebtoon",
     "newgrounds",
-    "ngomik",
     "nhentai",
     "nijie",
+    "nitter",
     "nozomi",
     "nsfwalbum",
+    "nudecollect",
     "paheal",
     "patreon",
     "philomena",
@@ -106,6 +116,7 @@ modules = [
     "pixiv",
     "pixnet",
     "plurk",
+    "poipiku",
     "pornhub",
     "pururin",
     "reactor",
@@ -123,9 +134,11 @@ modules = [
     "slickpic",
     "slideshare",
     "smugmug",
+    "soundgasm",
     "speakerdeck",
     "subscribestar",
     "tapas",
+    "tcbscans",
     "telegraph",
     "toyhouse",
     "tsumino",
@@ -134,19 +147,23 @@ modules = [
     "twibooru",
     "twitter",
     "unsplash",
+    "uploadir",
     "vanillarock",
+    "vichan",
     "vk",
     "vsco",
     "wallhaven",
     "wallpapercave",
     "warosu",
     "weasyl",
+    "webmshare",
     "webtoons",
     "weibo",
     "wikiart",
-    "wikieat",
+    "wikifeet",
     "xhamster",
     "xvideos",
+    "zerochan",
     "booru",
     "moebooru",
     "foolfuuka",
