@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2021 Mike Fährmann
+# Copyright 2018-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extract images from https://www.behance.net/"""
+"""Extractors for https://www.behance.net/"""
 
 from .common import Extractor, Message
 from .. import text
@@ -17,9 +17,9 @@ class BehanceExtractor(Extractor):
     """Base class for behance extractors"""
     category = "behance"
     root = "https://www.behance.net"
+    request_interval = (2.0, 4.0)
 
     def items(self):
-        yield Message.Version, 1
         for gallery in self.galleries():
             gallery["_extractor"] = BehanceGalleryExtractor
             yield Message.Queue, gallery["url"], self._update(gallery)
@@ -85,7 +85,7 @@ class BehanceGalleryExtractor(BehanceExtractor):
         }),
         # 'video' modules (#1282)
         ("https://www.behance.net/gallery/101185577/COLCCI", {
-            "pattern": r"ytdl:https://adobeprod-a\.akamaihd\.net/",
+            "pattern": r"ytdl:https://cdn-prod-ccv\.adobe\.com/",
             "count": 3,
         }),
     )
@@ -99,7 +99,6 @@ class BehanceGalleryExtractor(BehanceExtractor):
         imgs = self.get_images(data)
         data["count"] = len(imgs)
 
-        yield Message.Version, 1
         yield Message.Directory, data
         for data["num"], (url, module) in enumerate(imgs, 1):
             data["module"] = module
@@ -121,8 +120,8 @@ class BehanceGalleryExtractor(BehanceExtractor):
         }
         page = self.request(url, cookies=cookies).text
 
-        data = json.loads(text.extract(
-            page, 'id="beconfig-store_state">', '</script>')[0])
+        data = json.loads(text.extr(
+            page, 'id="beconfig-store_state">', '</script>'))
         return self._update(data["project"]["project"])
 
     def get_images(self, data):
@@ -139,7 +138,7 @@ class BehanceGalleryExtractor(BehanceExtractor):
 
             elif mtype == "video":
                 page = self.request(module["src"]).text
-                url = text.extract(page, '<source src="', '"')[0]
+                url = text.extr(page, '<source src="', '"')
                 if text.ext_from_url(url) == "m3u8":
                     url = "ytdl:" + url
                 append((url, module))
@@ -152,8 +151,7 @@ class BehanceGalleryExtractor(BehanceExtractor):
             elif mtype == "embed":
                 embed = module.get("original_embed") or module.get("embed")
                 if embed:
-                    url = "ytdl:" + text.extract(embed, 'src="', '"')[0]
-                    append((url, module))
+                    append(("ytdl:" + text.extr(embed, 'src="', '"'), module))
 
         return result
 
@@ -217,236 +215,253 @@ class BehanceCollectionExtractor(BehanceExtractor):
 
         query = """
 query GetMoodboardItemsAndRecommendations(
-    $id: Int!
-    $firstItem: Int!
-    $afterItem: String
-    $shouldGetRecommendations: Boolean!
-    $shouldGetItems: Boolean!
-    $shouldGetMoodboardFields: Boolean!
-  ) {
-    viewer @include(if: $shouldGetMoodboardFields) {
-      isOptedOutOfRecommendations
-    }
-    moodboard(id: $id) {
-      ...moodboardFields @include(if: $shouldGetMoodboardFields)
+  $id: Int!
+  $firstItem: Int!
+  $afterItem: String
+  $shouldGetRecommendations: Boolean!
+  $shouldGetItems: Boolean!
+  $shouldGetMoodboardFields: Boolean!
+) {
+  viewer @include(if: $shouldGetMoodboardFields) {
+    isOptedOutOfRecommendations
+    isAdmin
+  }
+  moodboard(id: $id) {
+    ...moodboardFields @include(if: $shouldGetMoodboardFields)
 
-      items(first: $firstItem, after: $afterItem) @include(if: $shouldGetItems)
-      {
-        pageInfo {
-          endCursor
-          hasNextPage
-        }
-        nodes {
-          ...nodesFields
-        }
+    items(first: $firstItem, after: $afterItem) @include(if: $shouldGetItems) {
+      pageInfo {
+        endCursor
+        hasNextPage
       }
+      nodes {
+        ...nodesFields
+      }
+    }
 
-      recommendedItems(first: 80) @include(if: $shouldGetRecommendations) {
-        nodes {
-          ...nodesFields
-          fetchSource
-        }
+    recommendedItems(first: 80) @include(if: $shouldGetRecommendations) {
+      nodes {
+        ...nodesFields
+        fetchSource
       }
     }
   }
+}
 
-  fragment moodboardFields on Moodboard {
+fragment moodboardFields on Moodboard {
+  id
+  label
+  privacy
+  followerCount
+  isFollowing
+  projectCount
+  url
+  isOwner
+  owners {
     id
-    label
-    privacy
-    followerCount
+    displayName
+    url
+    firstName
+    location
+    locationUrl
     isFollowing
-    projectCount
-    url
-    isOwner
-    owners {
-      id
-      displayName
-      url
-      firstName
-      location
-      locationUrl
-      images {
-        size_50 {
-          url
-        }
-        size_100 {
-          url
-        }
-        size_115 {
-          url
-        }
-        size_230 {
-          url
-        }
-        size_138 {
-          url
-        }
-        size_276 {
-          url
-        }
-      }
-    }
-  }
-
-  fragment projectFields on Project {
-    id
-    isOwner
-    publishedOn
-    matureAccess
-    hasMatureContent
-    modifiedOn
-    name
-    url
-    isPrivate
-    slug
-    fields {
-      label
-    }
-    colors {
-      r
-      g
-      b
-    }
-    owners {
-      url
-      displayName
-      id
-      location
-      locationUrl
-      isProfileOwner
-      images {
-        size_50 {
-          url
-        }
-        size_100 {
-          url
-        }
-        size_115 {
-          url
-        }
-        size_230 {
-          url
-        }
-        size_138 {
-          url
-        }
-        size_276 {
-          url
-        }
-      }
-    }
-    covers {
-      size_original {
+    images {
+      size_50 {
         url
       }
-      size_max_808 {
-        url
-      }
-      size_808 {
-        url
-      }
-      size_404 {
-        url
-      }
-      size_202 {
-        url
-      }
-      size_230 {
+      size_100 {
         url
       }
       size_115 {
         url
       }
-    }
-    stats {
-      views {
-        all
+      size_230 {
+        url
       }
-      appreciations {
-        all
+      size_138 {
+        url
       }
-      comments {
-        all
+      size_276 {
+        url
       }
     }
   }
+}
 
-  fragment exifDataValueFields on exifDataValue {
+fragment projectFields on Project {
+  id
+  isOwner
+  publishedOn
+  matureAccess
+  hasMatureContent
+  modifiedOn
+  name
+  url
+  isPrivate
+  slug
+  license {
+    license
+    description
     id
     label
-    value
-    searchValue
+    url
+    text
+    images
   }
-
-  fragment nodesFields on MoodboardItem {
+  fields {
+    label
+  }
+  colors {
+    r
+    g
+    b
+  }
+  owners {
+    url
+    displayName
     id
-    entityType
-    width
-    height
-    flexWidth
-    flexHeight
+    location
+    locationUrl
+    isProfileOwner
+    isFollowing
     images {
-      size
+      size_50 {
+        url
+      }
+      size_100 {
+        url
+      }
+      size_115 {
+        url
+      }
+      size_230 {
+        url
+      }
+      size_138 {
+        url
+      }
+      size_276 {
+        url
+      }
+    }
+  }
+  covers {
+    size_original {
       url
     }
+    size_max_808 {
+      url
+    }
+    size_808 {
+      url
+    }
+    size_404 {
+      url
+    }
+    size_202 {
+      url
+    }
+    size_230 {
+      url
+    }
+    size_115 {
+      url
+    }
+  }
+  stats {
+    views {
+      all
+    }
+    appreciations {
+      all
+    }
+    comments {
+      all
+    }
+  }
+}
 
-    entity {
-      ... on Project {
+fragment exifDataValueFields on exifDataValue {
+  id
+  label
+  value
+  searchValue
+}
+
+fragment nodesFields on MoodboardItem {
+  id
+  entityType
+  width
+  height
+  flexWidth
+  flexHeight
+  images {
+    size
+    url
+  }
+
+  entity {
+    ... on Project {
+      ...projectFields
+    }
+
+    ... on ImageModule {
+      project {
         ...projectFields
       }
 
-      ... on ImageModule {
-        project {
-          ...projectFields
-        }
-
-        exifData {
-          lens {
-            ...exifDataValueFields
-          }
-          software {
-            ...exifDataValueFields
-          }
-          makeAndModel {
-            ...exifDataValueFields
-          }
-          focalLength {
-            ...exifDataValueFields
-          }
-          iso {
-            ...exifDataValueFields
-          }
-          location {
-            ...exifDataValueFields
-          }
-          flash {
-            ...exifDataValueFields
-          }
-          exposureMode {
-            ...exifDataValueFields
-          }
-          shutterSpeed {
-            ...exifDataValueFields
-          }
-          aperture {
-            ...exifDataValueFields
-          }
-        }
+      colors {
+        r
+        g
+        b
       }
 
-      ... on MediaCollectionComponent {
-        project {
-          ...projectFields
+      exifData {
+        lens {
+          ...exifDataValueFields
+        }
+        software {
+          ...exifDataValueFields
+        }
+        makeAndModel {
+          ...exifDataValueFields
+        }
+        focalLength {
+          ...exifDataValueFields
+        }
+        iso {
+          ...exifDataValueFields
+        }
+        location {
+          ...exifDataValueFields
+        }
+        flash {
+          ...exifDataValueFields
+        }
+        exposureMode {
+          ...exifDataValueFields
+        }
+        shutterSpeed {
+          ...exifDataValueFields
+        }
+        aperture {
+          ...exifDataValueFields
         }
       }
     }
+
+    ... on MediaCollectionComponent {
+      project {
+        ...projectFields
+      }
+    }
   }
+}
 """
         variables = {
             "afterItem": "MAo=",
             "firstItem": 40,
-            "id"       : self.collection_id,
+            "id"       : int(self.collection_id),
             "shouldGetItems"          : True,
             "shouldGetMoodboardFields": False,
             "shouldGetRecommendations": False,
