@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2014-2022 Mike Fährmann
+# Copyright 2014-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,6 +8,7 @@
 
 """Common classes and constants used by extractor modules."""
 
+import os
 import re
 import ssl
 import time
@@ -19,7 +20,7 @@ import requests
 import threading
 from requests.adapters import HTTPAdapter
 from .message import Message
-from .. import config, text, util, exception
+from .. import config, text, util, cache, exception
 
 
 class Extractor():
@@ -35,6 +36,7 @@ class Extractor():
     browser = None
     root = ""
     test = None
+    finalize = None
     request_interval = 0.0
     request_interval_min = 0.0
     request_timestamp = 0.0
@@ -43,7 +45,6 @@ class Extractor():
     def __init__(self, match):
         self.log = logging.getLogger(self.category)
         self.url = match.string
-        self.finalize = None
 
         if self.basecategory:
             self.config = self._config_shared
@@ -52,6 +53,7 @@ class Extractor():
         self._parentdir = ""
 
         self._write_pages = self.config("write-pages", False)
+        self._retry_codes = self.config("retry-codes")
         self._retries = self.config("retries", 4)
         self._timeout = self.config("timeout", 30)
         self._verify = self.config("verify", True)
@@ -63,6 +65,8 @@ class Extractor():
 
         if self._retries < 0:
             self._retries = float("inf")
+        if not self._retry_codes:
+            self._retry_codes = ()
 
         self._init_session()
         self._init_cookies()
@@ -102,12 +106,15 @@ class Extractor():
             values[:0] = config.accumulate((self.subcategory,), key, conf=conf)
         return values
 
-    def request(self, url, *, method="GET", session=None, retries=None,
-                encoding=None, fatal=True, notfound=None, **kwargs):
+    def request(self, url, *, method="GET", session=None,
+                retries=None, retry_codes=None, encoding=None,
+                fatal=True, notfound=None, **kwargs):
         if session is None:
             session = self.session
         if retries is None:
             retries = self._retries
+        if retry_codes is None:
+            retry_codes = self._retry_codes
         if "proxies" not in kwargs:
             kwargs["proxies"] = self._proxies
         if "timeout" not in kwargs:
@@ -121,8 +128,7 @@ class Extractor():
             seconds = (self._interval() -
                        (time.time() - Extractor.request_timestamp))
             if seconds > 0.0:
-                self.log.debug("Sleeping for %.5s seconds", seconds)
-                time.sleep(seconds)
+                self.sleep(seconds, "request")
 
         while True:
             try:
@@ -149,16 +155,16 @@ class Extractor():
 
                 msg = "'{} {}' for '{}'".format(code, response.reason, url)
                 server = response.headers.get("Server")
-                if server and server.startswith("cloudflare"):
-                    if code == 503 and \
-                            b"jschl-answer" in response.content:
-                        self.log.warning("Cloudflare IUAM challenge")
+                if server and server.startswith("cloudflare") and \
+                        code in (403, 503):
+                    content = response.content
+                    if b"_cf_chl_opt" in content or b"jschl-answer" in content:
+                        self.log.warning("Cloudflare challenge")
                         break
-                    if code == 403 and \
-                            b'name="captcha-bypass"' in response.content:
+                    if b'name="captcha-bypass"' in content:
                         self.log.warning("Cloudflare CAPTCHA")
                         break
-                if code < 500 and code != 429 and code != 430:
+                if code not in retry_codes and code < 500:
                     break
 
             finally:
@@ -167,8 +173,9 @@ class Extractor():
             self.log.debug("%s (%s/%s)", msg, tries, retries+1)
             if tries > retries:
                 break
-            time.sleep(
-                max(tries, self._interval()) if self._interval else tries)
+            self.sleep(
+                max(tries, self._interval()) if self._interval else tries,
+                "retry")
             tries += 1
 
         raise exception.HttpError(msg, response)
@@ -200,6 +207,11 @@ class Extractor():
             self.log.info("Waiting until %s for %s.", isotime, reason)
         time.sleep(seconds)
 
+    def sleep(self, seconds, reason):
+        self.log.debug("Sleeping %.2f seconds (%s)",
+                       seconds, reason)
+        time.sleep(seconds)
+
     def _get_auth_info(self):
         """Return authentication information as (username, password) tuple"""
         username = self.config("username")
@@ -224,7 +236,9 @@ class Extractor():
         headers.clear()
         ssl_options = ssl_ciphers = 0
 
-        browser = self.config("browser") or self.browser
+        browser = self.config("browser")
+        if browser is None:
+            browser = self.browser
         if browser and isinstance(browser, str):
             browser, _, platform = browser.lower().partition(":")
 
@@ -254,11 +268,19 @@ class Extractor():
                             ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1)
             ssl_ciphers = SSL_CIPHERS[browser]
         else:
-            headers["User-Agent"] = self.config("user-agent", (
-                "Mozilla/5.0 (Windows NT 10.0; Win64; x64; "
-                "rv:91.0) Gecko/20100101 Firefox/91.0"))
+            useragent = self.config("user-agent")
+            if useragent is None:
+                useragent = ("Mozilla/5.0 (Windows NT 10.0; Win64; x64; "
+                             "rv:102.0) Gecko/20100101 Firefox/102.0")
+            elif useragent == "browser":
+                useragent = _browser_useragent()
+            headers["User-Agent"] = useragent
             headers["Accept"] = "*/*"
             headers["Accept-Language"] = "en-US,en;q=0.5"
+
+        if BROTLI:
+            headers["Accept-Encoding"] = "gzip, deflate, br"
+        else:
             headers["Accept-Encoding"] = "gzip, deflate"
 
         custom_headers = self.config("headers")
@@ -302,27 +324,50 @@ class Extractor():
         if cookies:
             if isinstance(cookies, dict):
                 self._update_cookies_dict(cookies, self.cookiedomain)
+
             elif isinstance(cookies, str):
                 cookiefile = util.expand_path(cookies)
                 try:
                     with open(cookiefile) as fp:
-                        cookies = util.load_cookiestxt(fp)
+                        util.cookiestxt_load(fp, self._cookiejar)
                 except Exception as exc:
                     self.log.warning("cookies: %s", exc)
                 else:
-                    self._update_cookies(cookies)
+                    self.log.debug("Loading cookies from '%s'", cookies)
                     self._cookiefile = cookiefile
+
+            elif isinstance(cookies, (list, tuple)):
+                key = tuple(cookies)
+                cookiejar = _browser_cookies.get(key)
+
+                if cookiejar is None:
+                    from ..cookies import load_cookies
+                    cookiejar = self._cookiejar.__class__()
+                    try:
+                        load_cookies(cookiejar, cookies)
+                    except Exception as exc:
+                        self.log.warning("cookies: %s", exc)
+                    else:
+                        _browser_cookies[key] = cookiejar
+                else:
+                    self.log.debug("Using cached cookies from %s", key)
+
+                setcookie = self._cookiejar.set_cookie
+                for cookie in cookiejar:
+                    setcookie(cookie)
+
             else:
                 self.log.warning(
-                    "expected 'dict' or 'str' value for 'cookies' option, "
-                    "got '%s' (%s)", cookies.__class__.__name__, cookies)
+                    "Expected 'dict', 'list', or 'str' value for 'cookies' "
+                    "option, got '%s' (%s)",
+                    cookies.__class__.__name__, cookies)
 
     def _store_cookies(self):
         """Store the session's cookiejar in a cookies.txt file"""
         if self._cookiefile and self.config("cookies-update", True):
             try:
                 with open(self._cookiefile, "w") as fp:
-                    util.save_cookiestxt(fp, self._cookiejar)
+                    util.cookiestxt_store(fp, self._cookiejar)
             except OSError as exc:
                 self.log.warning("cookies: %s", exc)
 
@@ -451,13 +496,21 @@ class Extractor():
 
         fname = "{:>02}_{}".format(
             Extractor._dump_index,
-            Extractor._dump_sanitize('_', response.url)
-        )[:250]
+            Extractor._dump_sanitize('_', response.url),
+        )
+
+        if util.WINDOWS:
+            path = os.path.abspath(fname)[:255]
+        else:
+            path = fname[:251]
 
         try:
-            with open(fname + ".dump", 'wb') as fp:
+            with open(path + ".txt", 'wb') as fp:
                 util.dump_response(
-                    response, fp, headers=(self._write_pages == "all"))
+                    response, fp,
+                    headers=(self._write_pages in ("all", "ALL")),
+                    hide_auth=(self._write_pages != "ALL")
+                )
         except Exception as e:
             self.log.warning("Failed to dump HTTP request (%s: %s)",
                              e.__class__.__name__, e)
@@ -603,18 +656,21 @@ class BaseExtractor(Extractor):
 
     def __init__(self, match):
         if not self.category:
-            for index, group in enumerate(match.groups()):
-                if group is not None:
-                    if index:
-                        self.category, self.root = self.instances[index-1]
-                        if not self.root:
-                            self.root = text.root_from_url(match.group(0))
-                    else:
-                        self.root = group
-                        self.category = group.partition("://")[2]
-                    break
+            self._init_category(match)
         Extractor.__init__(self, match)
 
+    def _init_category(self, match):
+        for index, group in enumerate(match.groups()):
+            if group is not None:
+                if index:
+                    self.category, self.root = self.instances[index-1]
+                    if not self.root:
+                        self.root = text.root_from_url(match.group(0))
+                else:
+                    self.root = group
+                    self.category = group.partition("://")[2]
+                break
+
     @classmethod
     def update(cls, instances):
         extra_instances = config.get(("extractor",), cls.basecategory)
@@ -682,21 +738,57 @@ def _build_requests_adapter(ssl_options, ssl_ciphers, source_address):
     return adapter
 
 
+@cache.cache(maxage=86400)
+def _browser_useragent():
+    """Get User-Agent header from default browser"""
+    import webbrowser
+    import socket
+
+    server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+    server.bind(("127.0.0.1", 6414))
+    server.listen(1)
+
+    webbrowser.open("http://127.0.0.1:6414/user-agent")
+
+    client = server.accept()[0]
+    server.close()
+
+    for line in client.recv(1024).split(b"\r\n"):
+        key, _, value = line.partition(b":")
+        if key.strip().lower() == b"user-agent":
+            useragent = value.strip()
+            break
+    else:
+        useragent = b""
+
+    client.send(b"HTTP/1.1 200 OK\r\n\r\n" + useragent)
+    client.close()
+
+    return useragent.decode()
+
+
 _adapter_cache = {}
+_browser_cookies = {}
 
 
 HTTP_HEADERS = {
     "firefox": (
-        ("User-Agent", "Mozilla/5.0 ({}; rv:91.0) "
-                       "Gecko/20100101 Firefox/91.0"),
+        ("User-Agent", "Mozilla/5.0 ({}; rv:102.0) "
+                       "Gecko/20100101 Firefox/102.0"),
         ("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,"
-                   "image/avif,*/*;q=0.8"),
+                   "image/avif,image/webp,*/*;q=0.8"),
         ("Accept-Language", "en-US,en;q=0.5"),
-        ("Accept-Encoding", "gzip, deflate"),
+        ("Accept-Encoding", None),
         ("Referer", None),
+        ("DNT", "1"),
         ("Connection", "keep-alive"),
         ("Upgrade-Insecure-Requests", "1"),
         ("Cookie", None),
+        ("Sec-Fetch-Dest", "empty"),
+        ("Sec-Fetch-Mode", "no-cors"),
+        ("Sec-Fetch-Site", "same-origin"),
+        ("TE", "trailers"),
     ),
     "chrome": (
         ("Upgrade-Insecure-Requests", "1"),
@@ -705,7 +797,7 @@ HTTP_HEADERS = {
         ("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,"
                    "image/webp,image/apng,*/*;q=0.8"),
         ("Referer", None),
-        ("Accept-Encoding", "gzip, deflate"),
+        ("Accept-Encoding", None),
         ("Accept-Language", "en-US,en;q=0.9"),
         ("Cookie", None),
     ),
@@ -729,8 +821,7 @@ SSL_CIPHERS = {
         "AES128-GCM-SHA256:"
         "AES256-GCM-SHA384:"
         "AES128-SHA:"
-        "AES256-SHA:"
-        "DES-CBC3-SHA"
+        "AES256-SHA"
     ),
     "chrome": (
         "TLS_AES_128_GCM_SHA256:"
@@ -753,6 +844,24 @@ SSL_CIPHERS = {
 }
 
 
+urllib3 = requests.packages.urllib3
+
+# detect brotli support
+try:
+    BROTLI = urllib3.response.brotli is not None
+except AttributeError:
+    BROTLI = False
+
+# set (urllib3) warnings filter
+action = config.get((), "warnings", "default")
+if action:
+    try:
+        import warnings
+        warnings.simplefilter(action, urllib3.exceptions.HTTPWarning)
+    except Exception:
+        pass
+del action
+
 # Undo automatic pyOpenSSL injection by requests
 pyopenssl = config.get((), "pyopenssl", False)
 if not pyopenssl:
