@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2022 Mike Fährmann
+# Copyright 2015-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -118,15 +118,33 @@ class DeviantartExtractor(Extractor):
             if "flash" in deviation:
                 yield self.commit(deviation, deviation["flash"])
 
-            if "excerpt" in deviation and self.commit_journal:
-                journal = self.api.deviation_content(deviation["deviationid"])
-                if self.extra:
-                    deviation["_journal"] = journal["html"]
-                yield self.commit_journal(deviation, journal)
+            if self.commit_journal:
+                if "excerpt" in deviation:
+                    journal = self.api.deviation_content(
+                        deviation["deviationid"])
+                elif "body" in deviation:
+                    journal = {"html": deviation.pop("body")}
+                else:
+                    journal = None
+                if journal:
+                    if self.extra:
+                        deviation["_journal"] = journal["html"]
+                    yield self.commit_journal(deviation, journal)
 
-            if self.extra:
-                txt = (deviation.get("description", "") +
-                       deviation.get("_journal", ""))
+            if not self.extra:
+                continue
+
+            # ref: https://www.deviantart.com
+            #      /developers/http/v1/20210526/object/editor_text
+            # the value of "features" is a JSON string with forward
+            # slashes escaped
+            text_content = \
+                deviation["text_content"]["body"]["features"].replace(
+                    "\\/", "/") if "text_content" in deviation else None
+            for txt in (text_content, deviation.get("description"),
+                        deviation.get("_journal")):
+                if txt is None:
+                    continue
                 for match in DeviantartStashExtractor.pattern.finditer(txt):
                     url = text.ensure_http_scheme(match.group(0))
                     deviation["_extractor"] = DeviantartStashExtractor
@@ -139,10 +157,19 @@ class DeviantartExtractor(Extractor):
         """Adjust the contents of a Deviation-object"""
         if "index" not in deviation:
             try:
-                deviation["index"] = text.parse_int(
-                    deviation["url"].rpartition("-")[2])
+                if deviation["url"].startswith("https://sta.sh"):
+                    filename = deviation["content"]["src"].split("/")[5]
+                    deviation["index_base36"] = filename.partition("-")[0][1:]
+                    deviation["index"] = id_from_base36(
+                        deviation["index_base36"])
+                else:
+                    deviation["index"] = text.parse_int(
+                        deviation["url"].rpartition("-")[2])
             except KeyError:
                 deviation["index"] = 0
+                deviation["index_base36"] = "0"
+        if "index_base36" not in deviation:
+            deviation["index_base36"] = base36_from_id(deviation["index"])
 
         if self.user:
             deviation["username"] = self.user
@@ -159,13 +186,11 @@ class DeviantartExtractor(Extractor):
 
         if self.comments:
             deviation["comments"] = (
-                self.api.comments_deviation(deviation["deviationid"])
+                self.api.comments(deviation["deviationid"], target="deviation")
                 if deviation["stats"]["comments"] else ()
             )
 
         # filename metadata
-        alphabet = "0123456789abcdefghijklmnopqrstuvwxyz"
-        deviation["index_base36"] = util.bencode(deviation["index"], alphabet)
         sub = re.compile(r"\W").sub
         deviation["filename"] = "".join((
             sub("_", deviation["title"].lower()), "_by_",
@@ -242,9 +267,10 @@ class DeviantartExtractor(Extractor):
         html = journal["html"]
         if html.startswith("<style"):
             html = html.partition("</style>")[2]
+        head, _, tail = html.rpartition("<script")
         content = "\n".join(
             text.unescape(text.remove_html(txt))
-            for txt in html.rpartition("<script")[0].split("<br />")
+            for txt in (head or tail).split("<br />")
         )
         txt = JOURNAL_TEMPLATE_TEXT.format(
             title=deviation["title"],
@@ -391,8 +417,9 @@ class DeviantartUserExtractor(DeviantartExtractor):
         }),
         ("https://www.deviantart.com/shimoda7", {
             "options": (("include", "all"),),
-            "pattern": r"/shimoda7/(gallery(/scraps)?|posts|favourites)$",
-            "count": 4,
+            "pattern": r"/shimoda7/"
+                       r"(gallery(/scraps)?|posts(/statuses)?|favourites)$",
+            "count": 5,
         }),
         ("https://shimoda7.deviantart.com/"),
     )
@@ -403,6 +430,7 @@ class DeviantartUserExtractor(DeviantartExtractor):
             (DeviantartGalleryExtractor , base + "gallery"),
             (DeviantartScrapsExtractor  , base + "gallery/scraps"),
             (DeviantartJournalExtractor , base + "posts"),
+            (DeviantartStatusExtractor  , base + "posts/statuses"),
             (DeviantartFavoriteExtractor, base + "favourites"),
         ), ("gallery",))
 
@@ -735,6 +763,97 @@ class DeviantartJournalExtractor(DeviantartExtractor):
         return self.api.browse_user_journals(self.user, self.offset)
 
 
+class DeviantartStatusExtractor(DeviantartExtractor):
+    """Extractor for an artist's status updates"""
+    subcategory = "status"
+    directory_fmt = ("{category}", "{username}", "Status")
+    filename_fmt = "{category}_{index}_{title}_{date}.{extension}"
+    archive_fmt = "S_{_username}_{index}.{extension}"
+    pattern = BASE_PATTERN + r"/posts/statuses"
+    test = (
+        ("https://www.deviantart.com/t1na/posts/statuses", {
+            "count": 0,
+        }),
+        ("https://www.deviantart.com/justgalym/posts/statuses", {
+            "count": 4,
+            "url": "bf4c44c0c60ff2648a880f4c3723464ad3e7d074",
+        }),
+        # shared deviation
+        ("https://www.deviantart.com/justgalym/posts/statuses", {
+            "options": (("journals", "none"),),
+            "count": 1,
+            "pattern": r"https://images-wixmp-\w+\.wixmp\.com/f"
+                       r"/[^/]+/[^.]+\.jpg\?token=",
+        }),
+        # shared sta.sh item
+        ("https://www.deviantart.com/vanillaghosties/posts/statuses", {
+            "options": (("journals", "none"), ("original", False)),
+            "range": "5-",
+            "count": 1,
+            "keyword": {
+                "index"       : int,
+                "index_base36": "re:^[0-9a-z]+$",
+                "url"         : "re:^https://sta.sh",
+            },
+        }),
+        ("https://www.deviantart.com/justgalym/posts/statuses", {
+            "options": (("journals", "text"),),
+            "url": "c8744f7f733a3029116607b826321233c5ca452d",
+        }),
+    )
+
+    def deviations(self):
+        for status in self.api.user_statuses(self.user, self.offset):
+            yield from self.status(status)
+
+    def status(self, status):
+        for item in status.get("items") or ():  # do not trust is_share
+            # shared deviations/statuses
+            if "deviation" in item:
+                yield item["deviation"].copy()
+            if "status" in item:
+                yield from self.status(item["status"].copy())
+        # assume is_deleted == true means necessary fields are missing
+        if status["is_deleted"]:
+            self.log.warning(
+                "Skipping status %s (deleted)", status.get("statusid"))
+            return
+        yield status
+
+    def prepare(self, deviation):
+        if "deviationid" in deviation:
+            return DeviantartExtractor.prepare(self, deviation)
+
+        try:
+            path = deviation["url"].split("/")
+            deviation["index"] = text.parse_int(path[-1] or path[-2])
+        except KeyError:
+            deviation["index"] = 0
+
+        if self.user:
+            deviation["username"] = self.user
+            deviation["_username"] = self.user.lower()
+        else:
+            deviation["username"] = deviation["author"]["username"]
+            deviation["_username"] = deviation["username"].lower()
+
+        deviation["date"] = dt = text.parse_datetime(deviation["ts"])
+        deviation["published_time"] = int(util.datetime_to_timestamp(dt))
+
+        deviation["da_category"] = "Status"
+        deviation["category_path"] = "status"
+        deviation["is_downloadable"] = False
+        deviation["title"] = "Status Update"
+
+        comments_count = deviation.pop("comments_count", 0)
+        deviation["stats"] = {"comments": comments_count}
+        if self.comments:
+            deviation["comments"] = (
+                self.api.comments(deviation["statusid"], target="status")
+                if comments_count else ()
+            )
+
+
 class DeviantartPopularExtractor(DeviantartExtractor):
     """Extractor for popular deviations"""
     subcategory = "popular"
@@ -854,7 +973,11 @@ class DeviantartDeviationExtractor(DeviantartExtractor):
     """Extractor for single deviations"""
     subcategory = "deviation"
     archive_fmt = "g_{_username}_{index}.{extension}"
-    pattern = BASE_PATTERN + r"/(art|journal)/(?:[^/?#]+-)?(\d+)"
+    pattern = (BASE_PATTERN + r"/(art|journal)/(?:[^/?#]+-)?(\d+)"
+               r"|(?:https?://)?(?:www\.)?deviantart\.com/"
+               r"(?:view/|deviation/|view(?:-full)?\.php/*\?(?:[^#]+&)?id=)"
+               r"(\d+)"  # bare deviation ID without slug
+               r"|(?:https?://)?fav\.me/d([0-9a-z]+)")  # base36
     test = (
         (("https://www.deviantart.com/shimoda7/art/For-the-sake-10073852"), {
             "options": (("original", 0),),
@@ -896,19 +1019,13 @@ class DeviantartDeviationExtractor(DeviantartExtractor):
             "range": "2-",
             "count": 4,
         }),
-        # video
-        ("https://www.deviantart.com/chi-u/art/-VIDEO-Brushes-330774593", {
-            "pattern": r"https://wixmp-.+wixmp.com/v/mp4/.+\.720p\.\w+.mp4",
-            "keyword": {
-                "filename": r"re:_video____brushes_\w+_by_chi_u-d5gxnb5",
-                "extension": "mp4",
-                "target": {
-                    "duration": 306,
-                    "filesize": 19367585,
-                    "quality": "720p",
-                    "src": str,
-                },
-            }
+        # sta.sh URL from deviation["text_content"]["body"]["features"]
+        (("https://www.deviantart.com"
+          "/cimar-wildehopps/art/Honorary-Vixen-859809305"), {
+            "options": (("extra", 1),),
+            "pattern": ("text:<!DOCTYPE html>\n|" +
+                        DeviantartStashExtractor.pattern),
+            "count": 2,
         }),
         # journal
         ("https://www.deviantart.com/shimoda7/journal/ARTility-583755752", {
@@ -920,12 +1037,37 @@ class DeviantartDeviationExtractor(DeviantartExtractor):
             "url": "e2e0044bd255304412179b6118536dbd9bb3bb0e",
             "pattern": "text:<!DOCTYPE html>\n",
         }),
+        # /view/ URLs
+        ("https://deviantart.com/view/904858796/", {
+            "content": "8770ec40ad1c1d60f6b602b16301d124f612948f",
+        }),
+        ("http://www.deviantart.com/view/890672057", {
+            "content": "1497e13d925caeb13a250cd666b779a640209236",
+        }),
+        ("https://www.deviantart.com/view/706871727", {
+            "content": "3f62ae0c2fca2294ac28e41888ea06bb37c22c65",
+        }),
+        ("https://www.deviantart.com/view/1", {
+            "exception": exception.NotFoundError,
+        }),
+        # /deviation/ (#3558)
+        ("https://www.deviantart.com/deviation/817215762"),
+        # fav.me (#3558)
+        ("https://fav.me/ddijrpu", {
+            "count": 1,
+        }),
+        ("https://fav.me/dddd", {
+            "exception": exception.NotFoundError,
+        }),
         # old-style URLs
         ("https://shimoda7.deviantart.com"
          "/art/For-the-sake-of-a-memory-10073852"),
         ("https://myria-moon.deviantart.com"
          "/art/Aime-Moi-part-en-vadrouille-261986576"),
         ("https://zzz.deviantart.com/art/zzz-1234567890"),
+        # old /view/ URLs from the Wayback Machine
+        ("https://www.deviantart.com/view.php?id=14864502"),
+        ("http://www.deviantart.com/view-full.php?id=100842"),
     )
 
     skip = Extractor.skip
@@ -933,11 +1075,13 @@ class DeviantartDeviationExtractor(DeviantartExtractor):
     def __init__(self, match):
         DeviantartExtractor.__init__(self, match)
         self.type = match.group(3)
-        self.deviation_id = match.group(4)
+        self.deviation_id = \
+            match.group(4) or match.group(5) or id_from_base36(match.group(6))
 
     def deviations(self):
         url = "{}/{}/{}/{}".format(
-            self.root, self.user, self.type, self.deviation_id)
+            self.root, self.user or "u", self.type or "art", self.deviation_id)
+
         uuid = text.extract(self._limited_request(url).text,
                             '"deviationUuid\\":\\"', '\\')[0]
         if not uuid:
@@ -963,13 +1107,9 @@ class DeviantartScrapsExtractor(DeviantartExtractor):
     _warning = True
 
     def deviations(self):
-        eclipse_api = DeviantartEclipseAPI(self)
-        if self._warning:
-            DeviantartScrapsExtractor._warning = False
-            if not self._check_cookies(self.cookienames):
-                self.log.warning(
-                    "No session cookies set: Unable to fetch mature scraps.")
+        self.login()
 
+        eclipse_api = DeviantartEclipseAPI(self)
         for obj in eclipse_api.gallery_scraps(self.user, self.offset):
             deviation = obj["deviation"]
             deviation_uuid = eclipse_api.deviation_extended_fetch(
@@ -980,6 +1120,17 @@ class DeviantartScrapsExtractor(DeviantartExtractor):
 
             yield self.api.deviation(deviation_uuid)
 
+    def login(self):
+        """Login and obtain session cookies"""
+        if not self._check_cookies(self.cookienames):
+            username, password = self._get_auth_info()
+            if username:
+                self._update_cookies(_login_impl(self, username, password))
+            elif self._warning:
+                self.log.warning(
+                    "No session cookies set: Unable to fetch mature scraps.")
+            DeviantartScrapsExtractor._warning = False
+
 
 class DeviantartFollowingExtractor(DeviantartExtractor):
     """Extractor for user's watched users"""
@@ -1118,9 +1269,9 @@ class DeviantartOAuthAPI():
                   "mature_content": self.mature}
         return self._pagination_list(endpoint, params)
 
-    def comments_deviation(self, deviation_id, offset=0):
-        """Fetch comments posted on a deviation"""
-        endpoint = "/comments/deviation/" + deviation_id
+    def comments(self, id, target, offset=0):
+        """Fetch comments posted on a target"""
+        endpoint = "/comments/{}/{}".format(target, id)
         params = {"maxdepth": "5", "offset": offset, "limit": 50,
                   "mature_content": self.mature}
         return self._pagination_list(endpoint, params=params, key="thread")
@@ -1156,8 +1307,6 @@ class DeviantartOAuthAPI():
 
     def deviation_metadata(self, deviations):
         """ Fetch deviation metadata for a set of deviations"""
-        if not deviations:
-            return []
         endpoint = "/deviation/metadata?" + "&".join(
             "deviationids[{}]={}".format(num, deviation["deviationid"])
             for num, deviation in enumerate(deviations)
@@ -1193,6 +1342,12 @@ class DeviantartOAuthAPI():
         endpoint = "/user/profile/" + username
         return self._call(endpoint, fatal=False)
 
+    def user_statuses(self, username, offset=0):
+        """Yield status updates of a specific user"""
+        endpoint = "/user/statuses/"
+        params = {"username": username, "offset": offset, "limit": 50}
+        return self._pagination(endpoint, params)
+
     def user_friends_watch(self, username):
         """Watch a user"""
         endpoint = "/user/friends/watch/" + username
@@ -1319,10 +1474,12 @@ class DeviantartOAuthAPI():
                             "Private deviations detected! Run 'gallery-dl "
                             "oauth:deviantart' and follow the instructions to "
                             "be able to access them.")
-                if self.metadata:
-                    self._metadata(results)
-                if self.folders:
-                    self._folders(results)
+                # "statusid" cannot be used instead
+                if results and "deviationid" in results[0]:
+                    if self.metadata:
+                        self._metadata(results)
+                    if self.folders:
+                        self._folders(results)
             yield from results
 
             if not data["has_more"] and (
@@ -1489,13 +1646,58 @@ class DeviantartEclipseAPI():
         return token
 
 
-@cache(maxage=100*365*24*3600, keyarg=0)
+@cache(maxage=100*365*86400, keyarg=0)
 def _refresh_token_cache(token):
     if token and token[0] == "#":
         return None
     return token
 
 
+@cache(maxage=28*86400, keyarg=1)
+def _login_impl(extr, username, password):
+    extr.log.info("Logging in as %s", username)
+
+    url = "https://www.deviantart.com/users/login"
+    page = extr.request(url).text
+
+    data = {}
+    for item in text.extract_iter(page, '<input type="hidden" name="', '"/>'):
+        name, _, value = item.partition('" value="')
+        data[name] = value
+
+    challenge = data.get("challenge")
+    if challenge and challenge != "0":
+        extr.log.warning("Login requires solving a CAPTCHA")
+        extr.log.debug(challenge)
+
+    data["username"] = username
+    data["password"] = password
+    data["remember"] = "on"
+
+    extr.sleep(2.0, "login")
+    url = "https://www.deviantart.com/_sisu/do/signin"
+    response = extr.request(url, method="POST", data=data)
+
+    if not response.history:
+        raise exception.AuthenticationError()
+
+    return {
+        cookie.name: cookie.value
+        for cookie in extr.session.cookies
+    }
+
+
+def id_from_base36(base36):
+    return util.bdecode(base36, _ALPHABET)
+
+
+def base36_from_id(deviation_id):
+    return util.bencode(int(deviation_id), _ALPHABET)
+
+
+_ALPHABET = "0123456789abcdefghijklmnopqrstuvwxyz"
+
+
 ###############################################################################
 # Journal Formats #############################################################
 
