@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2016-2021 Mike Fährmann
+# Copyright 2016-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -39,9 +39,6 @@ class FoolslideExtractor(BaseExtractor):
 
 
 BASE_PATTERN = FoolslideExtractor.update({
-    "kireicake": {
-        "root": "https://reader.kireicake.com",
-    },
     "powermanga": {
         "root": "https://read.powermanga.org",
         "pattern": r"read(?:er)?\.powermanga\.org",
@@ -63,10 +60,6 @@ class FoolslideChapterExtractor(FoolslideExtractor):
     archive_fmt = "{id}"
     pattern = BASE_PATTERN + r"(/read/[^/?#]+/[a-z-]+/\d+/\d+(?:/\d+)?)"
     test = (
-        ("https://reader.kireicake.com/read/wonderland/en/1/1/", {
-            "url": "b2d36bc0bc67e4c461c3a4d6444a2fd339f5d07e",
-            "keyword": "9f80947920a325e33aea7f5cd69ea669171903b6",
-        }),
         (("https://read.powermanga.org"
           "/read/one_piece_digital_colour_comics/en/0/75/"), {
             "url": "854c5817f8f767e1bccd05fa9d58ffb5a4b09384",
@@ -113,7 +106,7 @@ class FoolslideChapterExtractor(FoolslideExtractor):
         })
 
     def images(self, page):
-        return json.loads(text.extract(page, "var pages = ", ";")[0])
+        return json.loads(text.extr(page, "var pages = ", ";"))
 
 
 class FoolslideMangaExtractor(FoolslideExtractor):
@@ -122,10 +115,6 @@ class FoolslideMangaExtractor(FoolslideExtractor):
     categorytransfer = True
     pattern = BASE_PATTERN + r"(/series/[^/?#]+)"
     test = (
-        ("https://reader.kireicake.com/series/wonderland/", {
-            "url": "d067b649af1cc88fa8c8b698fde04a10909fd169",
-            "keyword": "268f43772fb239888ca5c5f6a4f65f99ffb3eefb",
-        }),
         (("https://read.powermanga.org"
           "/series/one_piece_digital_colour_comics/"), {
             "count": ">= 1",
