@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2018-2020 Leonardo Taccari
-# Copyright 2018-2022 Mike Fährmann
+# Copyright 2018-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -12,8 +12,8 @@
 from .common import Extractor, Message
 from .. import text, util, exception
 from ..cache import cache, memcache
+import binascii
 import json
-import time
 import re
 
 BASE_PATTERN = r"(?:https?://)?(?:www\.)?instagram\.com"
@@ -44,14 +44,10 @@ class InstagramExtractor(Extractor):
     def items(self):
         self.login()
 
-        api = self.config("api")
-        if api is None or api == "auto":
-            api = InstagramRestAPI if self._logged_in else InstagramGraphqlAPI
-        elif api == "graphql":
-            api = InstagramGraphqlAPI
+        if self.config("api") == "graphql":
+            self.api = InstagramGraphqlAPI(self)
         else:
-            api = InstagramRestAPI
-        self.api = api(self)
+            self.api = InstagramRestAPI(self)
 
         data = self.metadata()
         videos = self.config("videos", True)
@@ -69,6 +65,10 @@ class InstagramExtractor(Extractor):
 
             post["count"] = len(files)
             yield Message.Directory, post
+
+            if "date" in post:
+                del post["date"]
+
             for file in files:
                 file.update(post)
 
@@ -90,6 +90,11 @@ class InstagramExtractor(Extractor):
     def posts(self):
         return ()
 
+    def finalize(self):
+        if self._cursor:
+            self.log.info("Use '-o cursor=%s' to continue downloading "
+                          "from the current position", self._cursor)
+
     def request(self, url, **kwargs):
         response = Extractor.request(self, url, **kwargs)
 
@@ -97,10 +102,6 @@ class InstagramExtractor(Extractor):
 
             url = response.url
             if "/accounts/login/" in url:
-                if self._username:
-                    self.log.debug("Invalidating cached login session for "
-                                   "'%s'", self._username)
-                    _login_impl.invalidate(self._username)
                 page = "login"
             elif "/challenge/" in url:
                 page = "challenge"
@@ -108,9 +109,6 @@ class InstagramExtractor(Extractor):
                 page = None
 
             if page:
-                if self._cursor:
-                    self.log.info("Use '-o cursor=%s' to continue downloading "
-                                  "from the current position", self._cursor)
                 raise exception.StopExtraction("HTTP redirect to %s page (%s)",
                                                page, url.partition("?")[0])
 
@@ -118,14 +116,16 @@ class InstagramExtractor(Extractor):
         if www_claim is not None:
             self.www_claim = www_claim
 
+        csrf_token = response.cookies.get("csrftoken")
+        if csrf_token:
+            self.csrf_token = csrf_token
+
         return response
 
     def login(self):
-        self._username = None
         if not self._check_cookies(self.cookienames):
             username, password = self._get_auth_info()
             if username:
-                self._username = username
                 self._update_cookies(_login_impl(self, username, password))
             else:
                 self._logged_in = False
@@ -171,6 +171,15 @@ class InstagramExtractor(Extractor):
                 data["location_url"] = "{}/explore/locations/{}/{}/".format(
                     self.root, location["pk"], slug)
 
+            coauthors = post.get("coauthor_producers")
+            if coauthors:
+                data["coauthors"] = [
+                    {"id"       : user["pk"],
+                     "username" : user["username"],
+                     "full_name": user["full_name"]}
+                    for user in coauthors
+                ]
+
             if "carousel_media" in post:
                 items = post["carousel_media"]
                 data["sidecar_media_id"] = data["post_id"]
@@ -265,6 +274,14 @@ class InstagramExtractor(Extractor):
             data["location_url"] = "{}/explore/locations/{}/{}/".format(
                 self.root, location["id"], location["slug"])
 
+        coauthors = post.get("coauthor_producers")
+        if coauthors:
+            data["coauthors"] = [
+                {"id"      : user["id"],
+                 "username": user["username"]}
+                for user in coauthors
+            ]
+
         data["_files"] = files = []
         if "edge_sidecar_to_children" in post:
             for num, edge in enumerate(
@@ -338,6 +355,14 @@ class InstagramExtractor(Extractor):
                                          "username" : user["username"],
                                          "full_name": user["full_name"]})
 
+    def _init_cursor(self):
+        return self.config("cursor") or None
+
+    def _update_cursor(self, cursor):
+        self.log.debug("Cursor: %s", cursor)
+        self._cursor = cursor
+        return cursor
+
 
 class InstagramUserExtractor(InstagramExtractor):
     """Extractor for an Instagram user profile"""
@@ -353,12 +378,12 @@ class InstagramUserExtractor(InstagramExtractor):
         base = "{}/{}/".format(self.root, self.item)
         stories = "{}/stories/{}/".format(self.root, self.item)
         return self._dispatch_extractors((
+            (InstagramAvatarExtractor    , base + "avatar/"),
             (InstagramStoriesExtractor   , stories),
             (InstagramHighlightsExtractor, base + "highlights/"),
             (InstagramPostsExtractor     , base + "posts/"),
             (InstagramReelsExtractor     , base + "reels/"),
             (InstagramTaggedExtractor    , base + "tagged/"),
-            (InstagramChannelExtractor   , base + "channel/"),
         ), ("posts",))
 
 
@@ -409,8 +434,8 @@ class InstagramTaggedExtractor(InstagramExtractor):
             self.user_id = self.item[3:]
             return {"tagged_owner_id": self.user_id}
 
-        user = self.api.user(self.item)
-        self.user_id = user["id"]
+        self.user_id = self.api.user_id(self.item)
+        user = self.api.user_by_name(self.item)
 
         return {
             "tagged_owner_id" : user["id"],
@@ -422,18 +447,25 @@ class InstagramTaggedExtractor(InstagramExtractor):
         return self.api.user_tagged(self.user_id)
 
 
-class InstagramChannelExtractor(InstagramExtractor):
-    """Extractor for an Instagram user's channel posts"""
-    subcategory = "channel"
-    pattern = USER_PATTERN + r"/channel"
-    test = ("https://www.instagram.com/instagram/channel/", {
+class InstagramGuideExtractor(InstagramExtractor):
+    """Extractor for an Instagram guide"""
+    subcategory = "guide"
+    pattern = USER_PATTERN + r"/guide/[^/?#]+/(\d+)"
+    test = (("https://www.instagram.com/kadakaofficial/guide"
+             "/knit-i-need-collection/18131821684305217/"), {
         "range": "1-16",
         "count": ">= 16",
     })
 
+    def __init__(self, match):
+        InstagramExtractor.__init__(self, match)
+        self.guide_id = match.group(2)
+
+    def metadata(self):
+        return {"guide": self.api.guide(self.guide_id)}
+
     def posts(self):
-        uid = self.api.user_id(self.item)
-        return self.api.user_clips(uid)
+        return self.api.guide_media(self.guide_id)
 
 
 class InstagramSavedExtractor(InstagramExtractor):
@@ -475,25 +507,32 @@ class InstagramStoriesExtractor(InstagramExtractor):
     """Extractor for Instagram stories"""
     subcategory = "stories"
     pattern = (r"(?:https?://)?(?:www\.)?instagram\.com"
-               r"/stories/(?:highlights/(\d+)|([^/?#]+)(?:/(\d+))?)")
+               r"/s(?:tories/(?:highlights/(\d+)|([^/?#]+)(?:/(\d+))?)"
+               r"|/(aGlnaGxpZ2h0[^?#]+)(?:\?story_media_id=(\d+))?)")
     test = (
         ("https://www.instagram.com/stories/instagram/"),
         ("https://www.instagram.com/stories/highlights/18042509488170095/"),
         ("https://instagram.com/stories/geekmig/2724343156064789461"),
+        ("https://www.instagram.com/s/aGlnaGxpZ2h0OjE4MDQyNTA5NDg4MTcwMDk1"),
+        ("https://www.instagram.com/s/aGlnaGxpZ2h0OjE4MDQyNTA5NDg4MTcwMDk1"
+         "?story_media_id=2724343156064789461"),
     )
 
     def __init__(self, match):
-        self.highlight_id, self.user, self.media_id = match.groups()
-        if self.highlight_id:
+        h1, self.user, m1, h2, m2 = match.groups()
+
+        if self.user:
+            self.highlight_id = None
+        else:
             self.subcategory = InstagramHighlightsExtractor.subcategory
+            self.highlight_id = ("highlight:" + h1 if h1 else
+                                 binascii.a2b_base64(h2).decode())
+
+        self.media_id = m1 or m2
         InstagramExtractor.__init__(self, match)
 
     def posts(self):
-        if self.highlight_id:
-            reel_id = "highlight:" + self.highlight_id
-        else:
-            reel_id = self.api.user_id(self.user)
-
+        reel_id = self.highlight_id or self.api.user_id(self.user)
         reels = self.api.reels_media(reel_id)
 
         if self.media_id and reels:
@@ -536,6 +575,48 @@ class InstagramTagExtractor(InstagramExtractor):
         return self.api.tags_media(self.item)
 
 
+class InstagramAvatarExtractor(InstagramExtractor):
+    """Extractor for an Instagram user's avatar"""
+    subcategory = "avatar"
+    pattern = USER_PATTERN + r"/avatar"
+    test = ("https://www.instagram.com/instagram/avatar", {
+        "pattern": r"https://instagram\.[\w.-]+\.fbcdn\.net/v/t51\.2885-19"
+                   r"/281440578_1088265838702675_6233856337905829714_n\.jpg",
+    })
+
+    def posts(self):
+        if self._logged_in:
+            user_id = self.api.user_id(self.item, check_private=False)
+            user = self.api.user_by_id(user_id)
+            avatar = (user.get("hd_profile_pic_url_info") or
+                      user["hd_profile_pic_versions"][-1])
+        else:
+            user = self.item
+            if user.startswith("id:"):
+                user = self.api.user_by_id(user[3:])
+            else:
+                user = self.api.user_by_name(user)
+                user["pk"] = user["id"]
+            url = user.get("profile_pic_url_hd") or user["profile_pic_url"]
+            avatar = {"url": url, "width": 0, "height": 0}
+
+        pk = user.get("profile_pic_id")
+        if pk:
+            pk = pk.partition("_")[0]
+            code = shortcode_from_id(pk)
+        else:
+            pk = code = "avatar:" + str(user["pk"])
+
+        return ({
+            "pk"        : pk,
+            "code"      : code,
+            "user"      : user,
+            "caption"   : None,
+            "like_count": 0,
+            "image_versions2": {"candidates": (avatar,)},
+        },)
+
+
 class InstagramPostExtractor(InstagramExtractor):
     """Extractor for an Instagram post"""
     subcategory = "post"
@@ -647,6 +728,15 @@ class InstagramRestAPI():
     def __init__(self, extractor):
         self.extractor = extractor
 
+    def guide(self, guide_id):
+        endpoint = "/v1/guides/web_info/"
+        params = {"guide_id": guide_id}
+        return self._call(endpoint, params=params)
+
+    def guide_media(self, guide_id):
+        endpoint = "/v1/guides/guide/{}/".format(guide_id)
+        return self._pagination_guides(endpoint)
+
     def highlights_media(self, user_id):
         chunk_size = 5
         reel_ids = [hl["id"] for hl in self.highlights_tray(user_id)]
@@ -685,19 +775,37 @@ class InstagramRestAPI():
         return self._pagination_sections(endpoint, data)
 
     @memcache(keyarg=1)
-    def user(self, screen_name):
+    def user_by_name(self, screen_name):
         endpoint = "/v1/users/web_profile_info/"
         params = {"username": screen_name}
         return self._call(endpoint, params=params)["data"]["user"]
 
-    def user_id(self, screen_name):
+    def user_by_id(self, user_id):
+        endpoint = "/v1/users/{}/info/".format(user_id)
+        return self._call(endpoint)["user"]
+
+    def user_id(self, screen_name, check_private=True):
         if screen_name.startswith("id:"):
             return screen_name[3:]
-        return self.user(screen_name)["id"]
+        user = self.user_by_name(screen_name)
+        if user is None:
+            raise exception.AuthorizationError(
+                "Login required to access this profile")
+        if check_private and user["is_private"] and \
+                not user["followed_by_viewer"]:
+            name = user["username"]
+            s = "" if name.endswith("s") else "s"
+            raise exception.StopExtraction("%s'%s posts are private", name, s)
+        return user["id"]
 
     def user_clips(self, user_id):
         endpoint = "/v1/clips/user/"
-        data = {"target_user_id": user_id, "page_size": "50"}
+        data = {
+            "target_user_id": user_id,
+            "page_size": "50",
+            "max_id": None,
+            "include_feed_video": "true",
+        }
         return self._pagination_post(endpoint, data)
 
     def user_collection(self, collection_id):
@@ -723,24 +831,26 @@ class InstagramRestAPI():
     def _call(self, endpoint, **kwargs):
         extr = self.extractor
 
-        url = "https://i.instagram.com/api" + endpoint
+        url = "https://www.instagram.com/api" + endpoint
         kwargs["headers"] = {
+            "Accept"          : "*/*",
             "X-CSRFToken"     : extr.csrf_token,
             "X-Instagram-AJAX": "1006242110",
             "X-IG-App-ID"     : "936619743392459",
             "X-ASBD-ID"       : "198387",
             "X-IG-WWW-Claim"  : extr.www_claim,
-            "Origin"          : extr.root,
+            "X-Requested-With": "XMLHttpRequest",
+            "Alt-Used"        : "www.instagram.com",
             "Referer"         : extr.root + "/",
         }
-        kwargs["cookies"] = {
-            "csrftoken": extr.csrf_token,
-        }
         return extr.request(url, **kwargs).json()
 
     def _pagination(self, endpoint, params=None, media=False):
         if params is None:
             params = {}
+        extr = self.extractor
+        params["max_id"] = extr._init_cursor()
+
         while True:
             data = self._call(endpoint, params=params)
 
@@ -751,10 +861,13 @@ class InstagramRestAPI():
                 yield from data["items"]
 
             if not data.get("more_available"):
-                return
-            params["max_id"] = data["next_max_id"]
+                return extr._update_cursor(None)
+            params["max_id"] = extr._update_cursor(data["next_max_id"])
 
     def _pagination_post(self, endpoint, params):
+        extr = self.extractor
+        params["max_id"] = extr._init_cursor()
+
         while True:
             data = self._call(endpoint, method="POST", data=params)
 
@@ -763,33 +876,55 @@ class InstagramRestAPI():
 
             info = data["paging_info"]
             if not info.get("more_available"):
-                return
-            params["max_id"] = info["max_id"]
+                return extr._update_cursor(None)
+            params["max_id"] = extr._update_cursor(info["max_id"])
 
     def _pagination_sections(self, endpoint, params):
+        extr = self.extractor
+        params["max_id"] = extr._init_cursor()
+
         while True:
             info = self._call(endpoint, method="POST", data=params)
 
             yield from info["sections"]
 
             if not info.get("more_available"):
-                return
-            params["max_id"] = info["next_max_id"]
+                return extr._update_cursor(None)
             params["page"] = info["next_page"]
+            params["max_id"] = extr._update_cursor(info["next_max_id"])
+
+    def _pagination_guides(self, endpoint):
+        extr = self.extractor
+        params = {"max_id": extr._init_cursor()}
+
+        while True:
+            data = self._call(endpoint, params=params)
+
+            for item in data["items"]:
+                yield from item["media_items"]
+
+            if "next_max_id" not in data:
+                return extr._update_cursor(None)
+            params["max_id"] = extr._update_cursor(data["next_max_id"])
 
 
 class InstagramGraphqlAPI():
 
     def __init__(self, extractor):
         self.extractor = extractor
-        self.user = InstagramRestAPI(extractor).user
         self.user_collection = self.user_saved = self.reels_media = \
-            self.highlights_media = self._login_required
+            self.highlights_media = self.guide = self.guide_media = \
+            self._unsupported
         self._json_dumps = json.JSONEncoder(separators=(",", ":")).encode
 
+        api = InstagramRestAPI(extractor)
+        self.user_by_name = api.user_by_name
+        self.user_by_id = api.user_by_id
+        self.user_id = api.user_id
+
     @staticmethod
-    def _login_required(_=None):
-        raise exception.AuthorizationError("Login required")
+    def _unsupported(_=None):
+        raise exception.StopExtraction("Unsupported with GraphQL API")
 
     def highlights_tray(self, user_id):
         query_hash = "d4d88dc1500312af6f937f7b804c68c3"
@@ -824,11 +959,6 @@ class InstagramGraphqlAPI():
         return self._pagination(query_hash, variables,
                                 "hashtag", "edge_hashtag_to_media")
 
-    def user_id(self, screen_name):
-        if screen_name.startswith("id:"):
-            return screen_name[3:]
-        return self.user(screen_name)["id"]
-
     def user_clips(self, user_id):
         query_hash = "bc78b344a68ed16dd5d7f264681c4c76"
         variables = {"id": user_id, "first": 50}
@@ -862,18 +992,12 @@ class InstagramGraphqlAPI():
             "X-Requested-With": "XMLHttpRequest",
             "Referer"         : extr.root + "/",
         }
-        cookies = {
-            "csrftoken": extr.csrf_token,
-        }
-        return extr.request(
-            url, params=params, headers=headers, cookies=cookies,
-        ).json()["data"]
+        return extr.request(url, params=params, headers=headers).json()["data"]
 
     def _pagination(self, query_hash, variables,
                     key_data="user", key_edge=None):
-        cursor = self.extractor.config("cursor")
-        if cursor:
-            variables["after"] = cursor
+        extr = self.extractor
+        variables["after"] = extr._init_cursor()
 
         while True:
             data = self._call(query_hash, variables)[key_data]
@@ -884,57 +1008,20 @@ class InstagramGraphqlAPI():
 
             info = data["page_info"]
             if not info["has_next_page"]:
-                return
+                return extr._update_cursor(None)
             elif not data["edges"]:
                 s = "" if self.item.endswith("s") else "s"
                 raise exception.StopExtraction(
                     "%s'%s posts are private", self.item, s)
 
-            variables["after"] = self._cursor = info["end_cursor"]
-            self.extractor.log.debug("Cursor: %s", self._cursor)
+            variables["after"] = extr._update_cursor(info["end_cursor"])
 
 
-@cache(maxage=360*24*3600, keyarg=1)
+@cache(maxage=90*24*3600, keyarg=1)
 def _login_impl(extr, username, password):
-    extr.log.info("Logging in as %s", username)
-
-    url = extr.root + "/accounts/login/"
-    page = extr.request(url).text
-
-    headers = {
-        "X-Web-Device-Id" : text.extract(page, '"device_id":"', '"')[0],
-        "X-IG-App-ID"     : "936619743392459",
-        "X-ASBD-ID"       : "437806",
-        "X-IG-WWW-Claim"  : "0",
-        "X-Requested-With": "XMLHttpRequest",
-        "Referer"         : url,
-    }
-    url = extr.root + "/data/shared_data/"
-    data = extr.request(url, headers=headers).json()
-
-    headers["X-CSRFToken"] = data["config"]["csrf_token"]
-    headers["X-Instagram-AJAX"] = data["rollout_hash"]
-    headers["Origin"] = extr.root
-    data = {
-        "username"     : username,
-        "enc_password" : "#PWD_INSTAGRAM_BROWSER:0:{}:{}".format(
-            int(time.time()), password),
-        "queryParams"         : "{}",
-        "optIntoOneTap"       : "false",
-        "stopDeletionNonce"   : "",
-        "trustedDeviceRecords": "{}",
-    }
-    url = extr.root + "/accounts/login/ajax/"
-    response = extr.request(url, method="POST", headers=headers, data=data)
-
-    if not response.json().get("authenticated"):
-        raise exception.AuthenticationError()
-
-    cget = extr.session.cookies.get
-    return {
-        name: cget(name)
-        for name in ("sessionid", "mid", "ig_did")
-    }
+    extr.log.error("Login with username & password is no longer supported. "
+                   "Use browser cookies instead.")
+    return {}
 
 
 def id_from_shortcode(shortcode):
