@@ -20,9 +20,9 @@ class LolisafeExtractor(BaseExtractor):
 
 
 BASE_PATTERN = LolisafeExtractor.update({
-    "zzzz" : {
-        "root": "https://zz.ht",
-        "pattern": r"zz\.(?:ht|fo)",
+    "xbunkr": {
+        "root": "https://xbunkr.com",
+        "pattern": r"xbunkr\.com",
     },
 })
 
@@ -31,15 +31,15 @@ class LolisafeAlbumExtractor(LolisafeExtractor):
     subcategory = "album"
     pattern = BASE_PATTERN + "/a/([^/?#]+)"
     test = (
-        ("https://zz.ht/a/lop7W6EZ", {
-            "pattern": r"https://z\.zz\.fo/(4anuY|ih560)\.png",
-            "count": 2,
+        ("https://xbunkr.com/a/TA0bu3F4", {
+            "pattern": r"https://media\.xbunkr\.com/[^.]+\.\w+",
+            "count": 861,
             "keyword": {
-                "album_id": "lop7W6EZ",
-                "album_name": "ferris",
-            },
+                "album_id": "TA0bu3F4",
+                "album_name": "Hannahowo Onlyfans Photos",
+            }
         }),
-        ("https://zz.fo/a/lop7W6EZ"),
+        ("https://xbunkr.com/a/GNQc2I5d"),
     )
 
     def __init__(self, match):
@@ -47,9 +47,9 @@ class LolisafeAlbumExtractor(LolisafeExtractor):
         self.album_id = match.group(match.lastindex)
 
         domain = self.config("domain")
-        if domain is None or domain == "auto":
+        if domain == "auto":
             self.root = text.root_from_url(match.group(0))
-        else:
+        elif domain:
             self.root = text.ensure_http_scheme(domain)
 
     def items(self):
@@ -58,11 +58,10 @@ class LolisafeAlbumExtractor(LolisafeExtractor):
         yield Message.Directory, data
         for data["num"], file in enumerate(files, 1):
             url = file["file"]
-            if "_fallback" in file:
-                data["_fallback"] = file["_fallback"]
-            text.nameext_from_url(url, data)
-            data["name"], sep, data["id"] = data["filename"].rpartition("-")
-            yield Message.Url, url, data
+            file.update(data)
+            text.nameext_from_url(url, file)
+            file["name"], sep, file["id"] = file["filename"].rpartition("-")
+            yield Message.Url, url, file
 
     def fetch_album(self, album_id):
         url = "{}/api/album/get/{}".format(self.root, album_id)
