@@ -46,10 +46,10 @@ class RedditExtractor(Extractor):
                         submission["created_utc"])
                     yield Message.Directory, submission
                     visited.add(submission["id"])
-                    url = submission["url"]
                     submission["num"] = 0
 
-                    if url.startswith("https://i.redd.it/"):
+                    url = submission["url"]
+                    if url and url.startswith("https://i.redd.it/"):
                         text.nameext_from_url(url, submission)
                         yield Message.Url, url, submission
 
@@ -69,6 +69,11 @@ class RedditExtractor(Extractor):
                                 submission["_ytdl_extra"] = {
                                     "title": submission["title"],
                                 }
+                                try:
+                                    url = (submission["secure_media"]
+                                           ["reddit_video"]["dash_url"])
+                                except (KeyError, TypeError):
+                                    pass
                             yield Message.Url, "ytdl:" + url, submission
 
                     elif not submission["is_self"]:
@@ -144,8 +149,8 @@ class RedditExtractor(Extractor):
 class RedditSubredditExtractor(RedditExtractor):
     """Extractor for URLs from subreddits on reddit.com"""
     subcategory = "subreddit"
-    pattern = (r"(?:https?://)?(?:\w+\.)?reddit\.com/r/"
-               r"([^/?#]+(?:/([a-z]+))?)/?(?:\?([^#]*))?(?:$|#)")
+    pattern = (r"(?:https?://)?(?:\w+\.)?reddit\.com"
+               r"(/r/[^/?#]+(?:/([a-z]+))?)/?(?:\?([^#]*))?(?:$|#)")
     test = (
         ("https://www.reddit.com/r/lavaporn/", {
             "range": "1-20",
@@ -168,6 +173,20 @@ class RedditSubredditExtractor(RedditExtractor):
         return self.api.submissions_subreddit(self.subreddit, self.params)
 
 
+class RedditHomeExtractor(RedditSubredditExtractor):
+    """Extractor for submissions from your home feed on reddit.com"""
+    subcategory = "home"
+    pattern = (r"(?:https?://)?(?:\w+\.)?reddit\.com"
+               r"((?:/([a-z]+))?)/?(?:\?([^#]*))?(?:$|#)")
+    test = (
+        ("https://www.reddit.com/", {
+            "range": "1-20",
+            "count": ">= 20",
+        }),
+        ("https://old.reddit.com/top/?sort=top&t=month"),
+    )
+
+
 class RedditUserExtractor(RedditExtractor):
     """Extractor for URLs from posts by a reddit user"""
     subcategory = "user"
@@ -321,7 +340,7 @@ class RedditAPI():
 
     def submissions_subreddit(self, subreddit, params):
         """Collect all (submission, comments)-tuples of a subreddit"""
-        endpoint = "/r/" + subreddit + "/.json"
+        endpoint = subreddit + "/.json"
         params["limit"] = 100
         return self._pagination(endpoint, params)
 
@@ -409,7 +428,7 @@ class RedditAPI():
 
     def _pagination(self, endpoint, params):
         id_min = self._parse_id("id-min", 0)
-        id_max = self._parse_id("id-max", 2147483647)
+        id_max = self._parse_id("id-max", float("inf"))
         date_min, date_max = self.extractor._get_date_min_max(0, 253402210800)
 
         while True:
