@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2016-2022 Mike Fährmann
+# Copyright 2016-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -26,6 +26,7 @@ class TwitterExtractor(Extractor):
     cookiedomain = ".twitter.com"
     cookienames = ("auth_token",)
     root = "https://twitter.com"
+    browser = "firefox"
 
     def __init__(self, match):
         Extractor.__init__(self, match)
@@ -39,6 +40,7 @@ class TwitterExtractor(Extractor):
         self.videos = self.config("videos", True)
         self.cards = self.config("cards", False)
         self.cards_blacklist = self.config("cards-blacklist")
+        self.syndication = self.config("syndication")
         self._user = self._user_obj = None
         self._user_cache = {}
         self._init_sizes()
@@ -75,11 +77,6 @@ class TwitterExtractor(Extractor):
             else:
                 data = tweet
 
-            if seen_tweets is not None:
-                if data["id_str"] in seen_tweets:
-                    continue
-                seen_tweets.add(data["id_str"])
-
             if not self.retweets and "retweeted_status_id_str" in data:
                 self.log.debug("Skipping %s (retweet)", data["id_str"])
                 continue
@@ -97,6 +94,13 @@ class TwitterExtractor(Extractor):
                 self.log.debug("Skipping %s (reply)", data["id_str"])
                 continue
 
+            if seen_tweets is not None:
+                if data["id_str"] in seen_tweets:
+                    self.log.debug(
+                        "Skipping %s (previously seen)", data["id_str"])
+                    continue
+                seen_tweets.add(data["id_str"])
+
             files = []
             if "extended_entities" in data:
                 self._extract_media(
@@ -220,14 +224,16 @@ class TwitterExtractor(Extractor):
     def _extract_twitpic(self, tweet, files):
         for url in tweet["entities"].get("urls", ()):
             url = url["expanded_url"]
-            if "//twitpic.com/" in url and "/photos/" not in url:
-                response = self.request(url, fatal=False)
-                if response.status_code >= 400:
-                    continue
-                url = text.extr(
-                    response.text, 'name="twitter:image" value="', '"')
-                if url:
-                    files.append({"url": url})
+            if "//twitpic.com/" not in url or "/photos/" in url:
+                continue
+            if url.startswith("http:"):
+                url = "https" + url[4:]
+            response = self.request(url, fatal=False)
+            if response.status_code >= 400:
+                continue
+            url = text.extr(response.text, 'name="twitter:image" value="', '"')
+            if url:
+                files.append({"url": url})
 
     def _transform_tweet(self, tweet):
         if "author" in tweet:
@@ -299,6 +305,9 @@ class TwitterExtractor(Extractor):
 
         if "legacy" in user:
             user = user["legacy"]
+        elif "statuses_count" not in user and self.syndication == "extended":
+            # try to fetch extended user data
+            user = self.api.user_by_screen_name(user["screen_name"])["legacy"]
 
         uget = user.get
         entities = user["entities"]
@@ -361,18 +370,22 @@ class TwitterExtractor(Extractor):
     def _expand_tweets(self, tweets):
         seen = set()
         for tweet in tweets:
-
-            if "legacy" in tweet:
-                cid = tweet["legacy"]["conversation_id_str"]
-            else:
-                cid = tweet["conversation_id_str"]
-
-            if cid not in seen:
-                seen.add(cid)
-                try:
-                    yield from self.api.tweet_detail(cid)
-                except Exception:
-                    yield tweet
+            obj = tweet["legacy"] if "legacy" in tweet else tweet
+            cid = obj.get("conversation_id_str")
+            if not cid:
+                tid = obj["id_str"]
+                self.log.warning(
+                    "Unable to expand %s (no 'conversation_id')", tid)
+                continue
+            if cid in seen:
+                self.log.debug(
+                    "Skipping expansion of %s (previously seen)", cid)
+                continue
+            seen.add(cid)
+            try:
+                yield from self.api.tweet_detail(cid)
+            except Exception:
+                yield tweet
 
     def _make_tweet(self, user, id_str, url, timestamp):
         return {
@@ -417,7 +430,11 @@ class TwitterTimelineExtractor(TwitterExtractor):
             "url": "c570ac1aae38ed1463be726cc46f31cac3d82a40",
         }),
         # suspended account (#2216)
-        ("https://twitter.com/realDonaldTrump", {
+        ("https://twitter.com/OptionalTypo", {
+            "exception": exception.NotFoundError,
+        }),
+        # suspended account user ID
+        ("https://twitter.com/id:772949683521978368", {
             "exception": exception.NotFoundError,
         }),
         ("https://mobile.twitter.com/supernaturepics?p=i"),
@@ -629,7 +646,7 @@ class TwitterEventExtractor(TwitterExtractor):
     pattern = BASE_PATTERN + r"/i/events/(\d+)"
     test = ("https://twitter.com/i/events/1484669206993903616", {
         "range": "1-20",
-        "count": ">5",
+        "count": ">=1",
     })
 
     def metadata(self):
@@ -755,7 +772,7 @@ class TwitterTweetExtractor(TwitterExtractor):
         # retweet with missing media entities (#1555)
         ("https://twitter.com/morino_ya/status/1392763691599237121", {
             "options": (("retweets", True),),
-            "count": 4,
+            "count": 0,  # private
         }),
         # deleted quote tweet (#2225)
         ("https://twitter.com/i/web/status/1460044411165888515", {
@@ -768,7 +785,7 @@ class TwitterTweetExtractor(TwitterExtractor):
         # age-restricted (#2354)
         ("https://twitter.com/mightbecursed/status/1492954264909479936", {
             "options": (("syndication", True),),
-            "keywords": {"date": "dt:2022-02-13 20:10:09"},
+            "keyword": {"date": "dt:2022-02-13 20:10:09"},
             "count": 1,
         }),
         # media alt texts / descriptions (#2617)
@@ -778,7 +795,7 @@ class TwitterTweetExtractor(TwitterExtractor):
         # '?format=...&name=...'-style URLs
         ("https://twitter.com/poco_dandy/status/1150646424461176832", {
             "options": (("cards", True),),
-            "pattern": r"https://pbs.twimg.com/card_img/157\d+/\w+"
+            "pattern": r"https://pbs.twimg.com/card_img/157\d+/[\w-]+"
                        r"\?format=(jpg|png)&name=orig$",
             "range": "1-2",
         }),
@@ -882,7 +899,7 @@ class TwitterBackgroundExtractor(TwitterExtractor):
 
     def tweets(self):
         self.api._user_id_by_screen_name(self.user)
-        user = user = self._user_obj
+        user = self._user_obj
 
         try:
             url = user["legacy"]["profile_banner_url"]
@@ -929,16 +946,31 @@ class TwitterAPI():
     def __init__(self, extractor):
         self.extractor = extractor
 
-        self.root = "https://twitter.com/i/api"
+        self.root = "https://api.twitter.com"
+        cookies = extractor.session.cookies
+        cookiedomain = extractor.cookiedomain
+
+        csrf = extractor.config("csrf")
+        if csrf is None or csrf == "cookies":
+            csrf_token = cookies.get("ct0", domain=cookiedomain)
+        else:
+            csrf_token = None
+        if not csrf_token:
+            csrf_token = util.generate_token()
+            cookies.set("ct0", csrf_token, domain=cookiedomain)
+
+        auth_token = cookies.get("auth_token", domain=cookiedomain)
+
         self.headers = {
             "authorization": "Bearer AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejR"
                              "COuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu"
                              "4FA33AGWWjCpTnA",
             "x-guest-token": None,
-            "x-twitter-auth-type": None,
+            "x-twitter-auth-type": "OAuth2Session" if auth_token else None,
             "x-twitter-client-language": "en",
             "x-twitter-active-user": "yes",
-            "x-csrf-token": None,
+            "x-csrf-token": csrf_token,
+            "Origin": "https://twitter.com",
             "Referer": "https://twitter.com/",
         }
         self.params = {
@@ -951,24 +983,36 @@ class TwitterAPI():
             "include_can_dm": "1",
             "include_can_media_tag": "1",
             "include_ext_has_nft_avatar": "1",
+            "include_ext_is_blue_verified": "1",
+            "include_ext_verified_type": "1",
             "skip_status": "1",
             "cards_platform": "Web-12",
             "include_cards": "1",
             "include_ext_alt_text": "true",
+            "include_ext_limited_action_results": "false",
             "include_quote_count": "true",
             "include_reply_count": "1",
             "tweet_mode": "extended",
+            "include_ext_collab_control": "true",
+            "include_ext_views": "true",
             "include_entities": "true",
             "include_user_entities": "true",
             "include_ext_media_color": "true",
             "include_ext_media_availability": "true",
             "include_ext_sensitive_media_warning": "true",
+            "include_ext_trusted_friends_metadata": "true",
             "send_error_codes": "true",
             "simple_quoted_tweet": "true",
+            "q": None,
             "count": "100",
+            "query_source": None,
             "cursor": None,
-            "ext": "mediaStats,highlightedLabel,hasNftAvatar,"
-                   "voiceInfo,superFollowMetadata",
+            "pc": None,
+            "spelling_corrections": None,
+            "include_ext_edit_control": "true",
+            "ext": "mediaStats,highlightedLabel,hasNftAvatar,voiceInfo,"
+                   "enrichments,superFollowMetadata,unmentionInfo,editControl,"
+                   "collab_control,vibe",
         }
         self.variables = {
             "includePromotedContent": False,
@@ -987,31 +1031,9 @@ class TwitterAPI():
         }
 
         self._nsfw_warning = True
-        self._syndication = extractor.config("syndication")
+        self._syndication = self.extractor.syndication
         self._json_dumps = json.JSONEncoder(separators=(",", ":")).encode
 
-        cookies = extractor.session.cookies
-        cookiedomain = extractor.cookiedomain
-
-        csrf = extractor.config("csrf")
-        if csrf is None or csrf == "cookies":
-            csrf_token = cookies.get("ct0", domain=cookiedomain)
-        else:
-            csrf_token = None
-        if not csrf_token:
-            csrf_token = util.generate_token()
-            cookies.set("ct0", csrf_token, domain=cookiedomain)
-        self.headers["x-csrf-token"] = csrf_token
-
-        if cookies.get("auth_token", domain=cookiedomain):
-            # logged in
-            self.headers["x-twitter-auth-type"] = "OAuth2Session"
-        else:
-            # guest
-            guest_token = self._guest_token()
-            cookies.set("gt", guest_token, domain=cookiedomain)
-            self.headers["x-guest-token"] = guest_token
-
     def tweet_detail(self, tweet_id):
         endpoint = "/graphql/ItejhtHVxU7ksltgMmyaLA/TweetDetail"
         variables = {
@@ -1149,39 +1171,44 @@ class TwitterAPI():
         return self._call(endpoint, params)["data"]["user"]["result"]
 
     def _user_id_by_screen_name(self, screen_name):
-        if screen_name.startswith("id:"):
-            user_id = screen_name[3:]
-            user = self.user_by_rest_id(user_id)
-
-        else:
-            user = ()
-            try:
+        user = ()
+        try:
+            if screen_name.startswith("id:"):
+                user = self.user_by_rest_id(screen_name[3:])
+            else:
                 user = self.user_by_screen_name(screen_name)
-                user_id = user["rest_id"]
-            except KeyError:
-                if "unavailable_message" in user:
-                    raise exception.NotFoundError("{} ({})".format(
-                        user["unavailable_message"].get("text"),
-                        user.get("reason")), False)
-                else:
-                    raise exception.NotFoundError("user")
-
-        self.extractor._assign_user(user)
-        return user_id
+            self.extractor._assign_user(user)
+            return user["rest_id"]
+        except KeyError:
+            if "unavailable_message" in user:
+                raise exception.NotFoundError("{} ({})".format(
+                    user["unavailable_message"].get("text"),
+                    user.get("reason")), False)
+            else:
+                raise exception.NotFoundError("user")
 
     @cache(maxage=3600)
     def _guest_token(self):
-        root = "https://api.twitter.com"
         endpoint = "/1.1/guest/activate.json"
-        return str(self._call(endpoint, None, root, "POST")["guest_token"])
+        self.extractor.log.info("Requesting guest token")
+        return str(self._call(endpoint, None, "POST", False)["guest_token"])
 
-    def _call(self, endpoint, params, root=None, method="GET"):
-        if root is None:
-            root = self.root
+    def _authenticate_guest(self):
+        guest_token = self._guest_token()
+        if guest_token != self.headers["x-guest-token"]:
+            self.headers["x-guest-token"] = guest_token
+            self.extractor.session.cookies.set(
+                "gt", guest_token, domain=self.extractor.cookiedomain)
+
+    def _call(self, endpoint, params, method="GET", auth=True):
+        url = self.root + endpoint
 
         while True:
+            if not self.headers["x-twitter-auth-type"] and auth:
+                self._authenticate_guest()
+
             response = self.extractor.request(
-                root + endpoint, method=method, params=params,
+                url, method=method, params=params,
                 headers=self.headers, fatal=None)
 
             # update 'x-csrf-token' header (#1170)
@@ -1214,20 +1241,33 @@ class TwitterAPI():
 
     def _pagination_legacy(self, endpoint, params):
         original_retweets = (self.extractor.retweets == "original")
+        bottom = ("cursor-bottom-", "sq-cursor-bottom")
 
         while True:
-            cursor = tweet = None
             data = self._call(endpoint, params)
 
-            instr = data["timeline"]["instructions"]
-            if not instr:
+            instructions = data["timeline"]["instructions"]
+            if not instructions:
                 return
-            tweet_ids = []
+
             tweets = data["globalObjects"]["tweets"]
             users = data["globalObjects"]["users"]
+            tweet_id = cursor = None
+            tweet_ids = []
+            entries = ()
+
+            # process instructions
+            for instr in instructions:
+                if "addEntries" in instr:
+                    entries = instr["addEntries"]["entries"]
+                elif "replaceEntry" in instr:
+                    entry = instr["replaceEntry"]["entry"]
+                    if entry["entryId"].startswith(bottom):
+                        cursor = (entry["content"]["operation"]
+                                  ["cursor"]["value"])
 
             # collect tweet IDs and cursor value
-            for entry in instr[0]["addEntries"]["entries"]:
+            for entry in entries:
                 entry_startswith = entry["entryId"].startswith
 
                 if entry_startswith(("tweet-", "sq-I-t-")):
@@ -1239,11 +1279,11 @@ class TwitterAPI():
                         entry["content"]["timelineModule"]["metadata"]
                         ["conversationMetadata"]["allTweetIds"][::-1])
 
-                elif entry_startswith(("cursor-bottom-", "sq-cursor-bottom")):
+                elif entry_startswith(bottom):
                     cursor = entry["content"]["operation"]["cursor"]
                     if not cursor.get("stopOnEmptyResponse", True):
                         # keep going even if there are no tweets
-                        tweet = True
+                        tweet_id = True
                     cursor = cursor["value"]
 
                 elif entry_startswith("conversationThread-"):
@@ -1287,12 +1327,8 @@ class TwitterAPI():
                         quoted["quoted_by_id_str"] = tweet["id_str"]
                         yield quoted
 
-            # update cursor value
-            if "replaceEntry" in instr[-1] :
-                cursor = (instr[-1]["replaceEntry"]["entry"]
-                          ["content"]["operation"]["cursor"]["value"])
-
-            if not cursor or not tweet:
+            # stop on empty response
+            if not cursor or (not tweets and not tweet_id):
                 return
             params["cursor"] = cursor
 
@@ -1333,12 +1369,8 @@ class TwitterAPI():
                     if user.get("blocked_by"):
                         if self.headers["x-twitter-auth-type"] and \
                                 extr.config("logout"):
-                            guest_token = self._guest_token()
-                            extr.session.cookies.set(
-                                "gt", guest_token, domain=extr.cookiedomain)
                             extr._cookiefile = None
                             del extr.session.cookies["auth_token"]
-                            self.headers["x-guest-token"] = guest_token
                             self.headers["x-twitter-auth-type"] = None
                             extr.log.info("Retrying API request as guest")
                             continue
@@ -1515,6 +1547,12 @@ class TwitterAPI():
         else:
             retweet_id = None
 
+        # assume 'conversation_id' is the same as 'id' when the tweet
+        # is not a reply
+        if "conversation_id_str" not in tweet and \
+                "in_reply_to_status_id_str" not in tweet:
+            tweet["conversation_id_str"] = tweet["id_str"]
+
         tweet["created_at"] = text.parse_datetime(
             tweet["created_at"], "%Y-%m-%dT%H:%M:%S.%fZ").strftime(
             "%a %b %d %H:%M:%S +0000 %Y")
@@ -1559,8 +1597,6 @@ def _login_impl(extr, username, password):
             "Login with email is no longer possible. "
             "You need to provide your username or phone number instead.")
 
-    extr.log.info("Logging in as %s", username)
-
     def process(response):
         try:
             data = response.json()
@@ -1579,8 +1615,10 @@ def _login_impl(extr, username, password):
 
     extr.session.cookies.clear()
     api = TwitterAPI(extr)
+    api._authenticate_guest()
     headers = api.headers
-    headers["Referer"] = "https://twitter.com/i/flow/login"
+
+    extr.log.info("Logging in as %s", username)
 
     # init
     data = {
@@ -1634,7 +1672,7 @@ def _login_impl(extr, username, password):
             "web_modal": 1,
         },
     }
-    url = "https://twitter.com/i/api/1.1/onboarding/task.json?flow_name=login"
+    url = "https://api.twitter.com/1.1/onboarding/task.json?flow_name=login"
     response = extr.request(url, method="POST", headers=headers, json=data)
 
     data = {
@@ -1649,7 +1687,7 @@ def _login_impl(extr, username, password):
             },
         ],
     }
-    url = "https://twitter.com/i/api/1.1/onboarding/task.json"
+    url = "https://api.twitter.com/1.1/onboarding/task.json"
     response = extr.request(
         url, method="POST", headers=headers, json=data, fatal=None)
 
@@ -1673,7 +1711,7 @@ def _login_impl(extr, username, password):
             },
         ],
     }
-    #  url = "https://twitter.com/i/api/1.1/onboarding/task.json"
+    #  url = "https://api.twitter.com/1.1/onboarding/task.json"
     extr.sleep(random.uniform(2.0, 4.0), "login (username)")
     response = extr.request(
         url, method="POST", headers=headers, json=data, fatal=None)
@@ -1691,7 +1729,7 @@ def _login_impl(extr, username, password):
             },
         ],
     }
-    #  url = "https://twitter.com/i/api/1.1/onboarding/task.json"
+    #  url = "https://api.twitter.com/1.1/onboarding/task.json"
     extr.sleep(random.uniform(2.0, 4.0), "login (password)")
     response = extr.request(
         url, method="POST", headers=headers, json=data, fatal=None)
@@ -1708,7 +1746,7 @@ def _login_impl(extr, username, password):
             },
         ],
     }
-    #  url = "https://twitter.com/i/api/1.1/onboarding/task.json"
+    #  url = "https://api.twitter.com/1.1/onboarding/task.json"
     response = extr.request(
         url, method="POST", headers=headers, json=data, fatal=None)
     process(response)
