@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2021 Mike Fährmann
+# Copyright 2015-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,15 +8,14 @@
 
 import sys
 import json
-import time
 import errno
 import logging
-import operator
 import functools
 import collections
 from . import extractor, downloader, postprocessor
-from . import config, text, util, path, formatter, output, exception
+from . import config, text, util, path, formatter, output, exception, version
 from .extractor.message import Message
+from .output import stdout_write
 
 
 class Job():
@@ -33,12 +32,12 @@ class Job():
         self.pathfmt = None
         self.kwdict = {}
         self.status = 0
-        self.url_key = extr.config("url-metadata")
 
+        path_proxy = output.PathfmtProxy(self)
         self._logger_extra = {
             "job"      : self,
             "extractor": extr,
-            "path"     : output.PathfmtProxy(self),
+            "path"     : path_proxy,
             "keywords" : output.KwdictProxy(self),
         }
         extr.log = self._wrap_logger(extr.log)
@@ -54,13 +53,24 @@ class Job():
                 extr.category = pextr.category
                 extr.subcategory = pextr.subcategory
 
-            # reuse connection adapters
-            extr.session.adapters = pextr.session.adapters
+        self.metadata_url = extr.config("url-metadata")
+        self.metadata_http = extr.config("http-metadata")
+
+        version_info = extr.config("version-metadata")
+        metadata_path = extr.config("path-metadata")
 
         # user-supplied metadata
         kwdict = extr.config("keywords")
         if kwdict:
             self.kwdict.update(kwdict)
+        if metadata_path:
+            self.kwdict[metadata_path] = path_proxy
+        if version_info:
+            self.kwdict[version_info] = {
+                "version"         : version.__version__,
+                "is_executable"   : getattr(sys, "frozen", False),
+                "current_git_head": util.git_head()
+            }
 
         # predicates
         self.pred_url = self._prepare_predicates("image", True)
@@ -72,9 +82,10 @@ class Job():
         log = extractor.log
         msg = None
 
-        sleep = util.build_duration_func(extractor.config("sleep-extractor"))
+        sleep = util.build_duration_func(
+            extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep())
+            extractor.sleep(sleep(), "extractor")
 
         try:
             for msg in extractor:
@@ -118,8 +129,8 @@ class Job():
         """Call the appropriate message handler"""
         if msg[0] == Message.Url:
             _, url, kwdict = msg
-            if self.url_key:
-                kwdict[self.url_key] = url
+            if self.metadata_url:
+                kwdict[self.metadata_url] = url
             if self.pred_url(url, kwdict):
                 self.update_kwdict(kwdict)
                 self.handle_url(url, kwdict)
@@ -130,8 +141,8 @@ class Job():
 
         elif msg[0] == Message.Queue:
             _, url, kwdict = msg
-            if self.url_key:
-                kwdict[self.url_key] = url
+            if self.metadata_url:
+                kwdict[self.metadata_url] = url
             if self.pred_queue(url, kwdict):
                 self.handle_queue(url, kwdict)
 
@@ -152,6 +163,8 @@ class Job():
         extr = self.extractor
         kwdict["category"] = extr.category
         kwdict["subcategory"] = extr.subcategory
+        if self.metadata_http:
+            kwdict.pop(self.metadata_http, None)
         if self.kwdict:
             kwdict.update(self.kwdict)
 
@@ -201,7 +214,6 @@ class DownloadJob(Job):
     def __init__(self, url, parent=None):
         Job.__init__(self, url, parent)
         self.log = self.get_logger("download")
-        self.blacklist = None
         self.fallback = None
         self.archive = None
         self.sleep = None
@@ -209,6 +221,7 @@ class DownloadJob(Job):
         self.downloaders = {}
         self.out = output.select()
         self.visited = parent.visited if parent else set()
+        self._extractor_filter = None
         self._skipcnt = 0
 
     def handle_url(self, url, kwdict):
@@ -229,14 +242,17 @@ class DownloadJob(Job):
             self.handle_skip()
             return
 
-        if pathfmt.exists():
-            if archive:
-                archive.add(kwdict)
-            self.handle_skip()
-            return
+        if pathfmt.extension and not self.metadata_http:
+            pathfmt.build_path()
+
+            if pathfmt.exists():
+                if archive:
+                    archive.add(kwdict)
+                self.handle_skip()
+                return
 
         if self.sleep:
-            time.sleep(self.sleep())
+            self.extractor.sleep(self.sleep(), "download")
 
         # download from URL
         if not self.download(url):
@@ -268,7 +284,7 @@ class DownloadJob(Job):
 
         # download succeeded
         pathfmt.finalize()
-        self.out.success(pathfmt.path, 0)
+        self.out.success(pathfmt.path)
         self._skipcnt = 0
         if archive:
             archive.add(kwdict)
@@ -281,6 +297,9 @@ class DownloadJob(Job):
         if not self.pathfmt:
             self.initialize(kwdict)
         else:
+            if "post-after" in self.hooks:
+                for callback in self.hooks["post-after"]:
+                    callback(self.pathfmt)
             self.pathfmt.set_directory(kwdict)
         if "post" in self.hooks:
             for callback in self.hooks["post"]:
@@ -297,9 +316,9 @@ class DownloadJob(Job):
         else:
             extr = extractor.find(url)
             if extr:
-                if self.blacklist is None:
-                    self.blacklist = self._build_blacklist()
-                if extr.category in self.blacklist:
+                if self._extractor_filter is None:
+                    self._extractor_filter = self._build_extractor_filter()
+                if not self._extractor_filter(extr):
                     extr = None
 
         if extr:
@@ -335,14 +354,20 @@ class DownloadJob(Job):
             self._write_unsupported(url)
 
     def handle_finalize(self):
-        pathfmt = self.pathfmt
         if self.archive:
             self.archive.close()
+
+        pathfmt = self.pathfmt
         if pathfmt:
+            hooks = self.hooks
+            if "post-after" in hooks:
+                for callback in hooks["post-after"]:
+                    callback(pathfmt)
+
             self.extractor._store_cookies()
-            if "finalize" in self.hooks:
+            if "finalize" in hooks:
                 status = self.status
-                for callback in self.hooks["finalize"]:
+                for callback in hooks["finalize"]:
                     callback(pathfmt, status)
 
     def handle_skip(self):
@@ -393,8 +418,10 @@ class DownloadJob(Job):
 
     def initialize(self, kwdict=None):
         """Delayed initialization of PathFormat, etc."""
-        cfg = self.extractor.config
-        pathfmt = self.pathfmt = path.PathFormat(self.extractor)
+        extr = self.extractor
+        cfg = extr.config
+
+        pathfmt = self.pathfmt = path.PathFormat(extr)
         if kwdict:
             pathfmt.set_directory(kwdict)
 
@@ -407,17 +434,18 @@ class DownloadJob(Job):
         archive = cfg("archive")
         if archive:
             archive = util.expand_path(archive)
+            archive_format = (cfg("archive-prefix", extr.category) +
+                              cfg("archive-format", extr.archive_fmt))
             try:
                 if "{" in archive:
                     archive = formatter.parse(archive).format_map(kwdict)
-                self.archive = util.DownloadArchive(archive, self.extractor)
+                self.archive = util.DownloadArchive(archive, archive_format)
             except Exception as exc:
-                self.extractor.log.warning(
+                extr.log.warning(
                     "Failed to open download archive at '%s' ('%s: %s')",
                     archive, exc.__class__.__name__, exc)
             else:
-                self.extractor.log.debug(
-                    "Using download archive '%s'", archive)
+                extr.log.debug("Using download archive '%s'", archive)
 
         skip = cfg("skip", True)
         if skip:
@@ -439,27 +467,33 @@ class DownloadJob(Job):
             if self.archive:
                 self.archive.check = pathfmt.exists
 
-        postprocessors = self.extractor.config_accumulate("postprocessors")
+        if not cfg("postprocess", True):
+            return
+
+        postprocessors = extr.config_accumulate("postprocessors")
         if postprocessors:
             self.hooks = collections.defaultdict(list)
-            pp_log = self.get_logger("postprocessor")
-            pp_list = []
-            category = self.extractor.category
-            basecategory = self.extractor.basecategory
 
+            pp_log = self.get_logger("postprocessor")
             pp_conf = config.get((), "postprocessor") or {}
+            pp_opts = cfg("postprocessor-options")
+            pp_list = []
+
             for pp_dict in postprocessors:
                 if isinstance(pp_dict, str):
                     pp_dict = pp_conf.get(pp_dict) or {"name": pp_dict}
+                if pp_opts:
+                    pp_dict = pp_dict.copy()
+                    pp_dict.update(pp_opts)
 
-                whitelist = pp_dict.get("whitelist")
-                if whitelist and category not in whitelist and \
-                        basecategory not in whitelist:
-                    continue
-
-                blacklist = pp_dict.get("blacklist")
-                if blacklist and (
-                        category in blacklist or basecategory in blacklist):
+                clist = pp_dict.get("whitelist")
+                if clist is not None:
+                    negate = False
+                else:
+                    clist = pp_dict.get("blacklist")
+                    negate = True
+                if clist and not util.build_extractor_filter(
+                        clist, negate)(extr):
                     continue
 
                 name = pp_dict.get("name")
@@ -477,8 +511,7 @@ class DownloadJob(Job):
                     pp_list.append(pp_obj)
 
             if pp_list:
-                self.extractor.log.debug(
-                    "Active postprocessor modules: %s", pp_list)
+                extr.log.debug("Active postprocessor modules: %s", pp_list)
                 if "init" in self.hooks:
                     for callback in self.hooks["init"]:
                         callback(pathfmt)
@@ -500,38 +533,19 @@ class DownloadJob(Job):
         if condition(pathfmt.kwdict):
             callback(pathfmt)
 
-    def _build_blacklist(self):
-        wlist = self.extractor.config("whitelist")
-        if wlist is not None:
-            if isinstance(wlist, str):
-                wlist = wlist.split(",")
-
-            # build a set of all categories
-            blist = set()
-            add = blist.add
-            update = blist.update
-            get = operator.itemgetter(0)
-
-            for extr in extractor._list_classes():
-                category = extr.category
-                if category:
-                    add(category)
-                else:
-                    update(map(get, extr.instances))
-
-            # remove whitelisted categories
-            blist.difference_update(wlist)
-            return blist
-
-        blist = self.extractor.config("blacklist")
-        if blist is not None:
-            if isinstance(blist, str):
-                blist = blist.split(",")
-            blist = set(blist)
+    def _build_extractor_filter(self):
+        clist = self.extractor.config("whitelist")
+        if clist is not None:
+            negate = False
+            special = None
         else:
-            blist = {self.extractor.category}
-        blist |= util.SPECIAL_EXTRACTORS
-        return blist
+            clist = self.extractor.config("blacklist")
+            negate = True
+            special = util.SPECIAL_EXTRACTORS
+            if clist is None:
+                clist = (self.extractor.category,)
+
+        return util.build_extractor_filter(clist, negate, special)
 
 
 class SimulationJob(DownloadJob):
@@ -540,12 +554,11 @@ class SimulationJob(DownloadJob):
     def handle_url(self, url, kwdict):
         if not kwdict["extension"]:
             kwdict["extension"] = "jpg"
-        self.pathfmt.set_filename(kwdict)
-        self.out.skip(self.pathfmt.path)
         if self.sleep:
-            time.sleep(self.sleep())
+            self.extractor.sleep(self.sleep(), "download")
         if self.archive:
             self.archive.add(kwdict)
+        self.out.skip(self.pathfmt.build_filename(kwdict))
 
     def handle_directory(self, kwdict):
         if not self.pathfmt:
@@ -555,15 +568,24 @@ class SimulationJob(DownloadJob):
 class KeywordJob(Job):
     """Print available keywords"""
 
+    def __init__(self, url, parent=None):
+        Job.__init__(self, url, parent)
+        self.private = config.get(("output",), "private")
+
     def handle_url(self, url, kwdict):
-        print("\nKeywords for filenames and --filter:")
-        print("------------------------------------")
+        stdout_write("\nKeywords for filenames and --filter:\n"
+                     "------------------------------------\n")
+
+        if self.metadata_http and url.startswith("http"):
+            kwdict[self.metadata_http] = util.extract_headers(
+                self.extractor.request(url, method="HEAD"))
+
         self.print_kwdict(kwdict)
         raise exception.StopExtraction()
 
     def handle_directory(self, kwdict):
-        print("Keywords for directory names:")
-        print("-----------------------------")
+        stdout_write("Keywords for directory names:\n"
+                     "-----------------------------\n")
         self.print_kwdict(kwdict)
 
     def handle_queue(self, url, kwdict):
@@ -584,37 +606,50 @@ class KeywordJob(Job):
                 self.extractor.log.info(
                     "Try 'gallery-dl -K \"%s\"' instead.", url)
         else:
-            print("Keywords for --chapter-filter:")
-            print("------------------------------")
+            stdout_write("Keywords for --chapter-filter:\n"
+                         "------------------------------\n")
             self.print_kwdict(kwdict)
             if extr or self.extractor.categorytransfer:
-                print()
+                stdout_write("\n")
                 KeywordJob(extr or url, self).run()
         raise exception.StopExtraction()
 
-    @staticmethod
-    def print_kwdict(kwdict, prefix=""):
+    def print_kwdict(self, kwdict, prefix="", markers=None):
         """Print key-value pairs in 'kwdict' with formatting"""
+        write = sys.stdout.write
         suffix = "]" if prefix else ""
+
+        markerid = id(kwdict)
+        if markers is None:
+            markers = {markerid}
+        elif markerid in markers:
+            write("{}\n  <circular reference>\n".format(prefix[:-1]))
+            return  # ignore circular reference
+        else:
+            markers.add(markerid)
+
         for key, value in sorted(kwdict.items()):
-            if key[0] == "_":
+            if key[0] == "_" and not self.private:
                 continue
             key = prefix + key + suffix
 
             if isinstance(value, dict):
-                KeywordJob.print_kwdict(value, key + "[")
+                self.print_kwdict(value, key + "[", markers)
 
             elif isinstance(value, list):
-                if value and isinstance(value[0], dict):
-                    KeywordJob.print_kwdict(value[0], key + "[][")
+                if not value:
+                    pass
+                elif isinstance(value[0], dict):
+                    self.print_kwdict(value[0], key + "[N][", markers)
                 else:
-                    print(key, "[]", sep="")
-                    for val in value:
-                        print("  -", val)
+                    fmt = ("  {:>%s} {}\n" % len(str(len(value)))).format
+                    write(key + "[N]\n")
+                    for idx, val in enumerate(value, 0):
+                        write(fmt(idx, val))
 
             else:
                 # string or number
-                print(key, "\n  ", value, sep="")
+                write("{}\n  {}\n".format(key, value))
 
 
 class UrlJob(Job):
@@ -629,14 +664,14 @@ class UrlJob(Job):
 
     @staticmethod
     def handle_url(url, _):
-        print(url)
+        stdout_write(url + "\n")
 
     @staticmethod
     def handle_url_fallback(url, kwdict):
-        print(url)
+        stdout_write(url + "\n")
         if "_fallback" in kwdict:
             for url in kwdict["_fallback"]:
-                print("|", url)
+                stdout_write("| " + url + "\n")
 
     def handle_queue(self, url, kwdict):
         cls = kwdict.get("_extractor")
@@ -673,15 +708,18 @@ class InfoJob(Job):
         return 0
 
     def _print_multi(self, title, *values):
-        print(title, "\n  ", " / ".join(json.dumps(v) for v in values), sep="")
+        stdout_write("{}\n  {}\n\n".format(
+            title, " / ".join(json.dumps(v) for v in values)))
 
     def _print_config(self, title, optname, value):
         optval = self.extractor.config(optname, util.SENTINEL)
         if optval is not util.SENTINEL:
-            print(title, "(custom):\n ", json.dumps(optval))
-            print(title, "(default):\n ", json.dumps(value))
+            stdout_write(
+                "{} (custom):\n  {}\n{} (default):\n  {}\n\n".format(
+                    title, json.dumps(optval), title, json.dumps(value)))
         elif value:
-            print(title, "(default):\n ", json.dumps(value))
+            stdout_write(
+                "{} (default):\n  {}\n\n".format(title, json.dumps(value)))
 
 
 class DataJob(Job):
@@ -694,17 +732,18 @@ class DataJob(Job):
         self.ascii = config.get(("output",), "ascii", ensure_ascii)
 
         private = config.get(("output",), "private")
-        self.filter = util.identity if private else util.filter_dict
+        self.filter = dict.copy if private else util.filter_dict
 
     def run(self):
+        extractor = self.extractor
         sleep = util.build_duration_func(
-            self.extractor.config("sleep-extractor"))
+            extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep())
+            extractor.sleep(sleep(), "extractor")
 
         # collect data
         try:
-            for msg in self.extractor:
+            for msg in extractor:
                 self.dispatch(msg)
         except exception.StopExtraction:
             pass
