@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2022 Mike Fährmann
+# Copyright 2017-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -48,15 +48,18 @@ class DeprecatedConfigConstAction(argparse.Action):
 class ParseAction(argparse.Action):
     """Parse <key>=<value> options and set them as config values"""
     def __call__(self, parser, namespace, values, option_string=None):
-        key, _, value = values.partition("=")
-        try:
-            value = json.loads(value)
-        except ValueError:
-            pass
+        key, value = _parse_option(values)
         key = key.split(".")  # splitting an empty string becomes [""]
         namespace.options.append((key[:-1], key[-1], value))
 
 
+class OptionAction(argparse.Action):
+    """Parse <key>=<value> options for """
+    def __call__(self, parser, namespace, values, option_string=None):
+        key, value = _parse_option(values)
+        namespace.options_pp[key] = value
+
+
 class Formatter(argparse.HelpFormatter):
     """Custom HelpFormatter class to customize help output"""
     def __init__(self, *args, **kwargs):
@@ -73,6 +76,15 @@ class Formatter(argparse.HelpFormatter):
             return self._metavar_formatter(action, action.dest)(1)[0]
 
 
+def _parse_option(opt):
+    key, _, value = opt.partition("=")
+    try:
+        value = json.loads(value)
+    except ValueError:
+        pass
+    return key, value
+
+
 def build_parser():
     """Build and configure an ArgumentParser object"""
     parser = argparse.ArgumentParser(
@@ -142,10 +154,12 @@ def build_parser():
     )
     general.add_argument(
         "--cookies-from-browser",
-        dest="cookies_from_browser", metavar="BROWSER[+KEYRING][:PROFILE]",
+        dest="cookies_from_browser",
+        metavar="BROWSER[+KEYRING][:PROFILE][::CONTAINER]",
         help=("Name of the browser to load cookies from, "
-              "with optional keyring name prefixed with '+' and "
-              "profile prefixed with ':'"),
+              "with optional keyring name prefixed with '+', "
+              "profile prefixed with ':', and "
+              "container prefixed with '::' ('none' for no container)"),
     )
 
     output = parser.add_argument_group("Output Options")
@@ -323,7 +337,7 @@ def build_parser():
     configuration.add_argument(
         "--ignore-config",
         dest="load_config", action="store_false",
-        help="Do not read the default configuration files",
+        help="Do not read default configuration files",
     )
 
     authentication = parser.add_argument_group("Authentication Options")
@@ -347,7 +361,7 @@ def build_parser():
     selection.add_argument(
         "--download-archive",
         dest="archive", metavar="FILE", action=ConfigAction,
-        help=("Record all downloaded files in the archive file and "
+        help=("Record all downloaded or skipped files in FILE and "
               "skip downloading any file already in it"),
     )
     selection.add_argument(
@@ -365,19 +379,20 @@ def build_parser():
     selection.add_argument(
         "--range",
         dest="image-range", metavar="RANGE", action=ConfigAction,
-        help=("Index-range(s) specifying which images to download. "
-              "For example '5-10' or '1,3-5,10-'"),
+        help=("Index range(s) specifying which files to download. "
+              "These can be either a constant value, range, or slice "
+              "(e.g. '5', '8-20', or '1:24:3')"),
     )
     selection.add_argument(
         "--chapter-range",
         dest="chapter-range", metavar="RANGE", action=ConfigAction,
-        help=("Like '--range', but applies to manga-chapters "
+        help=("Like '--range', but applies to manga chapters "
               "and other delegated URLs"),
     )
     selection.add_argument(
         "--filter",
         dest="image-filter", metavar="EXPR", action=ConfigAction,
-        help=("Python expression controlling which images to download. "
+        help=("Python expression controlling which files to download. "
               "Files for which the expression evaluates to False are ignored. "
               "Available keys are the filename-specific ones listed by '-K'. "
               "Example: --filter \"image_width >= 1000 and "
@@ -386,7 +401,7 @@ def build_parser():
     selection.add_argument(
         "--chapter-filter",
         dest="chapter-filter", metavar="EXPR", action=ConfigAction,
-        help=("Like '--filter', but applies to manga-chapters "
+        help=("Like '--filter', but applies to manga chapters "
               "and other delegated URLs"),
     )
 
@@ -470,7 +485,7 @@ def build_parser():
         dest="postprocessors", metavar="CMD",
         action=AppendCommandAction, const={"name": "exec"},
         help=("Execute CMD for each downloaded file. "
-              "Example: --exec 'convert {} {}.png && rm {}'"),
+              "Example: --exec \"convert {} {}.png && rm {}\""),
     )
     postprocessor.add_argument(
         "--exec-after",
@@ -478,13 +493,18 @@ def build_parser():
         action=AppendCommandAction, const={
             "name": "exec", "event": "finalize"},
         help=("Execute CMD after all files were downloaded successfully. "
-              "Example: --exec-after 'cd {} && convert * ../doc.pdf'"),
+              "Example: --exec-after \"cd {} && convert * ../doc.pdf\""),
     )
     postprocessor.add_argument(
         "-P", "--postprocessor",
         dest="postprocessors", metavar="NAME", action="append",
         help="Activate the specified post processor",
     )
+    postprocessor.add_argument(
+        "-O", "--postprocessor-option",
+        dest="options_pp", metavar="OPT", action=OptionAction, default={},
+        help="Additional '<key>=<value>' post processor options",
+    )
 
     parser.add_argument(
         "urls",
