@@ -10,6 +10,7 @@
 
 from .common import PostProcessor
 from .. import util, formatter
+import sys
 import os
 
 
@@ -18,17 +19,37 @@ class MetadataPP(PostProcessor):
     def __init__(self, job, options):
         PostProcessor.__init__(self, job)
 
-        mode = options.get("mode", "json")
-        if mode == "custom":
+        mode = options.get("mode")
+        cfmt = options.get("content-format") or options.get("format")
+        omode = "w"
+        filename = None
+
+        if mode == "tags":
+            self.write = self._write_tags
+            ext = "txt"
+        elif mode == "modify":
+            self.run = self._run_modify
+            self.fields = {
+                name: formatter.parse(value, None, util.identity).format_map
+                for name, value in options.get("fields").items()
+            }
+            ext = None
+        elif mode == "delete":
+            self.run = self._run_delete
+            self.fields = options.get("fields")
+            ext = None
+        elif mode == "custom" or not mode and cfmt:
             self.write = self._write_custom
-            cfmt = options.get("content-format") or options.get("format")
             if isinstance(cfmt, list):
                 cfmt = "\n".join(cfmt) + "\n"
             self._content_fmt = formatter.parse(cfmt).format_map
             ext = "txt"
-        elif mode == "tags":
-            self.write = self._write_tags
-            ext = "txt"
+        elif mode == "jsonl":
+            self.write = self._write_json
+            self.indent = None
+            self.ascii = options.get("ascii", False)
+            omode = "a"
+            filename = "data.jsonl"
         else:
             self.write = self._write_json
             self.indent = options.get("indent", 4)
@@ -41,11 +62,14 @@ class MetadataPP(PostProcessor):
             sep = os.sep + (os.altsep or "")
             self._metadir = util.expand_path(directory).rstrip(sep) + os.sep
 
-        filename = options.get("filename")
+        filename = options.get("filename", filename)
         extfmt = options.get("extension-format")
         if filename:
-            self._filename = self._filename_custom
-            self._filename_fmt = formatter.parse(filename).format_map
+            if filename == "-":
+                self.run = self._run_stdout
+            else:
+                self._filename = self._filename_custom
+                self._filename_fmt = formatter.parse(filename).format_map
         elif extfmt:
             self._filename = self._filename_extfmt
             self._extension_fmt = formatter.parse(extfmt).format_map
@@ -82,6 +106,9 @@ class MetadataPP(PostProcessor):
             self.archive = None
 
         self.mtime = options.get("mtime")
+        self.omode = options.get("open", omode)
+        self.encoding = options.get("encoding", "utf-8")
+        self.private = options.get("private", False)
 
     def run(self, pathfmt):
         archive = self.archive
@@ -92,11 +119,11 @@ class MetadataPP(PostProcessor):
         path = directory + self._filename(pathfmt)
 
         try:
-            with open(path, "w", encoding="utf-8") as fp:
+            with open(path, self.omode, encoding=self.encoding) as fp:
                 self.write(fp, pathfmt.kwdict)
         except FileNotFoundError:
             os.makedirs(directory, exist_ok=True)
-            with open(path, "w", encoding="utf-8") as fp:
+            with open(path, self.omode, encoding=self.encoding) as fp:
                 self.write(fp, pathfmt.kwdict)
 
         if archive:
@@ -107,6 +134,35 @@ class MetadataPP(PostProcessor):
             if mtime:
                 util.set_mtime(path, mtime)
 
+    def _run_stdout(self, pathfmt):
+        self.write(sys.stdout, pathfmt.kwdict)
+
+    def _run_modify(self, pathfmt):
+        kwdict = pathfmt.kwdict
+        for key, func in self.fields.items():
+            obj = kwdict
+            try:
+                while "[" in key:
+                    name, _, key = key.partition("[")
+                    obj = obj[name]
+                    key = key.rstrip("]")
+                obj[key] = func(kwdict)
+            except Exception:
+                pass
+
+    def _run_delete(self, pathfmt):
+        kwdict = pathfmt.kwdict
+        for key in self.fields:
+            obj = kwdict
+            try:
+                while "[" in key:
+                    name, _, key = key.partition("[")
+                    obj = obj[name]
+                    key = key.rstrip("]")
+                del obj[key]
+            except Exception:
+                pass
+
     def _directory(self, pathfmt):
         return pathfmt.realdirectory
 
@@ -154,7 +210,9 @@ class MetadataPP(PostProcessor):
         fp.write("\n".join(tags) + "\n")
 
     def _write_json(self, fp, kwdict):
-        util.dump_json(util.filter_dict(kwdict), fp, self.ascii, self.indent)
+        if not self.private:
+            kwdict = util.filter_dict(kwdict)
+        util.dump_json(kwdict, fp, self.ascii, self.indent)
 
 
 __postprocessor__ = MetadataPP
