@@ -19,6 +19,7 @@ import binascii
 import datetime
 import functools
 import itertools
+import subprocess
 import urllib.parse
 from http.cookiejar import Cookie
 from email.utils import mktime_tz, parsedate_tz
@@ -161,13 +162,16 @@ def delete_items(obj, keys):
 
 def enumerate_reversed(iterable, start=0, length=None):
     """Enumerate 'iterable' and return its elements in reverse order"""
-    start -= 1
     if length is None:
         length = len(iterable)
-    return zip(
-        range(length - start, start, -1),
-        reversed(iterable),
-    )
+
+    try:
+        iterable = zip(range(start-1+length, start-1, -1), reversed(iterable))
+    except TypeError:
+        iterable = list(zip(range(start, start+length), iterable))
+        iterable.reverse()
+
+    return iterable
 
 
 def number_to_string(value, numbers=(int, float)):
@@ -227,7 +231,14 @@ Status: {response.status_code} {response.reason}
 Request Headers
 ---------------
 {request_headers}
-
+"""
+        if request.body:
+            outfmt += """
+Request Body
+------------
+{request.body}
+"""
+        outfmt += """
 Response Headers
 ----------------
 {response_headers}
@@ -270,6 +281,39 @@ Response Headers
         fp.write(response.content)
 
 
+def extract_headers(response):
+    headers = response.headers
+    data = dict(headers)
+
+    hcd = headers.get("content-disposition")
+    if hcd:
+        name = text.extr(hcd, 'filename="', '"')
+        if name:
+            text.nameext_from_url(name, data)
+
+    hlm = headers.get("last-modified")
+    if hlm:
+        data["date"] = datetime.datetime(*parsedate_tz(hlm)[:6])
+
+    return data
+
+
+@functools.lru_cache(maxsize=None)
+def git_head():
+    try:
+        out, err = subprocess.Popen(
+            ("git", "rev-parse", "--short", "HEAD"),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            cwd=os.path.dirname(os.path.abspath(__file__)),
+        ).communicate()
+        if out and not err:
+            return out.decode().rstrip()
+    except (OSError, subprocess.SubprocessError):
+        pass
+    return None
+
+
 def expand_path(path):
     """Expand environment variables and tildes (~)"""
     if not path:
@@ -302,9 +346,9 @@ def set_mtime(path, mtime):
         pass
 
 
-def load_cookiestxt(fp):
-    """Parse a Netscape cookies.txt file and return a list of its Cookies"""
-    cookies = []
+def cookiestxt_load(fp, cookiejar):
+    """Parse a Netscape cookies.txt file and add its Cookies to 'cookiejar'"""
+    set_cookie = cookiejar.set_cookie
 
     for line in fp:
 
@@ -321,11 +365,12 @@ def load_cookiestxt(fp):
 
         domain, domain_specified, path, secure, expires, name, value = \
             line.split("\t")
+
         if not name:
             name = value
             value = None
 
-        cookies.append(Cookie(
+        set_cookie(Cookie(
             0, name, value,
             None, False,
             domain,
@@ -337,12 +382,11 @@ def load_cookiestxt(fp):
             False, None, None, {},
         ))
 
-    return cookies
 
-
-def save_cookiestxt(fp, cookies):
+def cookiestxt_store(fp, cookies):
     """Write 'cookies' in Netscape cookies.txt format to 'fp'"""
-    fp.write("# Netscape HTTP Cookie File\n\n")
+    write = fp.write
+    write("# Netscape HTTP Cookie File\n\n")
 
     for cookie in cookies:
         if not cookie.domain:
@@ -355,15 +399,15 @@ def save_cookiestxt(fp, cookies):
             name = cookie.name
             value = cookie.value
 
-        fp.write("\t".join((
+        write("\t".join((
             cookie.domain,
             "TRUE" if cookie.domain.startswith(".") else "FALSE",
             cookie.path,
             "TRUE" if cookie.secure else "FALSE",
             "0" if cookie.expires is None else str(cookie.expires),
             name,
-            value,
-        )) + "\n")
+            value + "\n",
+        )))
 
 
 def code_to_language(code, default=None):
@@ -415,8 +459,84 @@ CODES = {
 }
 
 
-class UniversalNone():
-    """None-style object that supports more operations than None itself"""
+def parse_inputfile(file, log):
+    """Filter and process strings from an input file.
+
+    Lines starting with '#' and empty lines will be ignored.
+    Lines starting with '-' will be interpreted as a key-value pair separated
+      by an '='. where 'key' is a dot-separated option name and 'value' is a
+      JSON-parsable value. These configuration options will be applied while
+      processing the next URL.
+    Lines starting with '-G' are the same as above, except these options will
+      be applied for *all* following URLs, i.e. they are Global.
+    Everything else will be used as a potential URL.
+
+    Example input file:
+
+    # settings global options
+    -G base-directory = "/tmp/"
+    -G skip = false
+
+    # setting local options for the next URL
+    -filename="spaces_are_optional.jpg"
+    -skip    = true
+
+    https://example.org/
+
+    # next URL uses default filename and 'skip' is false.
+    https://example.com/index.htm # comment1
+    https://example.com/404.htm   # comment2
+    """
+    gconf = []
+    lconf = []
+    strip_comment = None
+
+    for line in file:
+        line = line.strip()
+
+        if not line or line[0] == "#":
+            # empty line or comment
+            continue
+
+        elif line[0] == "-":
+            # config spec
+            if len(line) >= 2 and line[1] == "G":
+                conf = gconf
+                line = line[2:]
+            else:
+                conf = lconf
+                line = line[1:]
+
+            key, sep, value = line.partition("=")
+            if not sep:
+                log.warning("input file: invalid <key>=<value> pair: %s", line)
+                continue
+
+            try:
+                value = json.loads(value.strip())
+            except ValueError as exc:
+                log.warning("input file: unable to parse '%s': %s", value, exc)
+                continue
+
+            key = key.strip().split(".")
+            conf.append((key[:-1], key[-1], value))
+
+        else:
+            # url
+            if " #" in line or "\t#" in line:
+                if strip_comment is None:
+                    strip_comment = re.compile(r"\s+#.*").sub
+                line = strip_comment("", line)
+            if gconf or lconf:
+                yield ExtendedUrl(line, gconf, lconf)
+                gconf = []
+                lconf = []
+            else:
+                yield line
+
+
+class CustomNone():
+    """None-style type that supports more operations than regular None"""
     __slots__ = ()
 
     def __getattribute__(self, _):
@@ -425,10 +545,28 @@ class UniversalNone():
     def __getitem__(self, _):
         return self
 
+    def __iter__(self):
+        return self
+
+    def __call__(self, *args, **kwargs):
+        return self
+
+    @staticmethod
+    def __next__():
+        raise StopIteration
+
     @staticmethod
     def __bool__():
         return False
 
+    @staticmethod
+    def __len__():
+        return 0
+
+    @staticmethod
+    def __format__(_):
+        return "None"
+
     @staticmethod
     def __str__():
         return "None"
@@ -436,7 +574,7 @@ class UniversalNone():
     __repr__ = __str__
 
 
-NONE = UniversalNone()
+NONE = CustomNone()
 EPOCH = datetime.datetime(1970, 1, 1)
 SECOND = datetime.timedelta(0, 1)
 WINDOWS = (os.name == "nt")
@@ -461,6 +599,8 @@ def compile_expression(expr, name="<expr>", globals=GLOBALS):
 
 def build_duration_func(duration, min=0.0):
     if not duration:
+        if min:
+            return lambda: min
         return None
 
     if isinstance(duration, str):
@@ -581,75 +721,72 @@ def chain_predicates(predicates, url, kwdict):
 
 
 class RangePredicate():
-    """Predicate; True if the current index is in the given range"""
+    """Predicate; True if the current index is in the given range(s)"""
+
     def __init__(self, rangespec):
-        self.ranges = self.optimize_range(self.parse_range(rangespec))
+        self.ranges = ranges = self._parse(rangespec)
         self.index = 0
 
-        if self.ranges:
-            self.lower, self.upper = self.ranges[0][0], self.ranges[-1][1]
+        if ranges:
+            # technically wrong, but good enough for now
+            # and evaluating min/max for a large range is slow
+            self.lower = min(r.start for r in ranges)
+            self.upper = max(r.stop for r in ranges) - 1
         else:
-            self.lower, self.upper = 0, 0
+            self.lower = self.upper = 0
 
-    def __call__(self, url, _):
-        self.index += 1
+    def __call__(self, _url, _kwdict):
+        self.index = index = self.index + 1
 
-        if self.index > self.upper:
+        if index > self.upper:
             raise exception.StopExtraction()
 
-        for lower, upper in self.ranges:
-            if lower <= self.index <= upper:
+        for range in self.ranges:
+            if index in range:
                 return True
         return False
 
     @staticmethod
-    def parse_range(rangespec):
+    def _parse(rangespec):
         """Parse an integer range string and return the resulting ranges
 
         Examples:
-            parse_range("-2,4,6-8,10-") -> [(1,2), (4,4), (6,8), (10,INTMAX)]
-            parse_range(" - 3 , 4-  4, 2-6") -> [(1,3), (4,4), (2,6)]
+            _parse("-2,4,6-8,10-")      -> [(1,3), (4,5), (6,9), (10,INTMAX)]
+            _parse(" - 3 , 4-  4, 2-6") -> [(1,4), (4,5), (2,7)]
+            _parse("1:2,4:8:2")         -> [(1,1), (4,7,2)]
         """
         ranges = []
+        append = ranges.append
 
-        for group in rangespec.split(","):
+        if isinstance(rangespec, str):
+            rangespec = rangespec.split(",")
+
+        for group in rangespec:
             if not group:
                 continue
-            first, sep, last = group.partition("-")
-            if not sep:
-                beg = end = int(first)
+
+            elif ":" in group:
+                start, _, stop = group.partition(":")
+                stop, _, step = stop.partition(":")
+                append(range(
+                    int(start) if start.strip() else 1,
+                    int(stop) if stop.strip() else sys.maxsize,
+                    int(step) if step.strip() else 1,
+                ))
+
+            elif "-" in group:
+                start, _, stop = group.partition("-")
+                append(range(
+                    int(start) if start.strip() else 1,
+                    int(stop) + 1 if stop.strip() else sys.maxsize,
+                ))
+
             else:
-                beg = int(first) if first.strip() else 1
-                end = int(last) if last.strip() else sys.maxsize
-            ranges.append((beg, end) if beg <= end else (end, beg))
+                start = int(group)
+                append(range(start, start+1))
 
         return ranges
 
-    @staticmethod
-    def optimize_range(ranges):
-        """Simplify/Combine a parsed list of ranges
-
-        Examples:
-            optimize_range([(2,4), (4,6), (5,8)]) -> [(2,8)]
-            optimize_range([(1,1), (2,2), (3,6), (8,9))]) -> [(1,6), (8,9)]
-        """
-        if len(ranges) <= 1:
-            return ranges
-
-        ranges.sort()
-        riter = iter(ranges)
-        result = []
-
-        beg, end = next(riter)
-        for lower, upper in riter:
-            if lower > end+1:
-                result.append((beg, end))
-                beg, end = lower, upper
-            elif upper > end:
-                end = upper
-        result.append((beg, end))
-        return result
-
 
 class UniquePredicate():
     """Predicate; True if given URL has not been encountered before"""
@@ -669,6 +806,8 @@ class FilterPredicate():
     """Predicate; True if evaluating the given expression returns True"""
 
     def __init__(self, expr, target="image"):
+        if not isinstance(expr, str):
+            expr = "(" + ") and (".join(expr) + ")"
         name = "<{} filter>".format(target)
         self.expr = compile_expression(expr, name)
 
@@ -693,21 +832,27 @@ class ExtendedUrl():
 class DownloadArchive():
 
     def __init__(self, path, format_string, cache_key="_archive_key"):
-        con = sqlite3.connect(path, timeout=60, check_same_thread=False)
+        try:
+            con = sqlite3.connect(path, timeout=60, check_same_thread=False)
+        except sqlite3.OperationalError:
+            os.makedirs(os.path.dirname(path))
+            con = sqlite3.connect(path, timeout=60, check_same_thread=False)
         con.isolation_level = None
 
         self.close = con.close
         self.cursor = con.cursor()
-        self.keygen = format_string.format_map
+
+        from . import formatter
+        self.keygen = formatter.parse(format_string).format_map
         self._cache_key = cache_key
 
         try:
             self.cursor.execute("CREATE TABLE IF NOT EXISTS archive "
-                                "(entry PRIMARY KEY) WITHOUT ROWID")
+                                "(entry TEXT PRIMARY KEY) WITHOUT ROWID")
         except sqlite3.OperationalError:
             # fallback for missing WITHOUT ROWID support (#553)
             self.cursor.execute("CREATE TABLE IF NOT EXISTS archive "
-                                "(entry PRIMARY KEY)")
+                                "(entry TEXT PRIMARY KEY)")
 
     def check(self, kwdict):
         """Return True if the item described by 'kwdict' exists in archive"""
@@ -720,4 +865,4 @@ class DownloadArchive():
         """Add item described by 'kwdict' to archive"""
         key = kwdict.get(self._cache_key) or self.keygen(kwdict)
         self.cursor.execute(
-            "INSERT OR IGNORE INTO archive VALUES (?)", (key,))
+            "INSERT OR IGNORE INTO archive (entry) VALUES (?)", (key,))
